function [finalModelDG finalModelCA individualResults] ...
    = getHippoCampus(image_name, extract_contours, show_output, refinement, ...
    writeTimeExcel,segmentationMethod,modelsToUse,imageMatrix,referenceToUse, ...
    metaheuristic, registration_type, SaveSegmentationEvolution, ...
    ValidateSegmentationResults, typeOfROI, metaheuristicRuns, performSegmentation, ...
    SOURCE_FOLDER, OUTPUT_FOLDER, REF_FOLDER, REF_CONT_FOLDER, ELAB_FOLDER, ...
    ELAB_CONT_FOLDER, FILENAME_TIME, numberModelsToTest, generations, populationSize, ...
    MODEL_TO_COMPARE,ImageModality)

%% parameters:
% - image_name: name of the image, in original size, the toolbox will extract the hippocampus. If extract_contours is 1 it has to be
%       in SOURCE_FOLDER. If extract_contours is 0, they has to be in ELAB_FOLDER and ELAB_CONT_FOLDER.
%       The name is considered without extension, the file in SOURCE_FOLDER is .jpg
% - extract_contours: 1 search in SOURCE_FOLDER and resize image and extract contours, 0 uses images that have already preprocessed
% - show_output: 0 no output, 1 output text, 2 output text and most
% relevant images, 3 all images
% - refinement: 0 no segmentation refinement, 1 segmentation refinement
%       using classifiers
% - writeTimeExcel: 0 no time output in Excel file, 1 output creating new file, 2 output appending the existent file
% - segmentationMethod: 'IO' or 'LS' or 'IOECL'
% - modelsToUse: they are the get_model function to use:
%       'classical','ASC','morePoints'
% - referenceToUse: is the number of reference to use. If it's 0 -> we use
%       registration, if it's different to 0 we use only the reference included.
%       If you include the reference to use you need to have the cropped
%       ROI in the folder images/source with this format:
%       'NAME'_ROI.png

%% return values:
% - bestReferenceSlice: best reference slice achieved by the system
% - finalModelDG and finalModelCA
% - individualResults: include different measurements, from totalTime (time of the whole segmentation process including read input
% image, contour extraction, global registration, local registration, best
% reference slice selection, preprocessing ROI, DG localization, CA
% localization, output localization, and segmentation) to TruePositivesPercentage, FalsePositivesPercentage,
% FalseNegativesPercentage (with both methods: IterOtsu and LevelSets, and with and without refinement)

% The outputs will be written in the following folders and files:
% - .\individualResults: the .mat file with the performance values (when this file is launched with Main_code.m)
% - file execution_times.xlsx in .\images with the execution parts of each
% part.
% - .\images\hippocampi: one folder for each segmented image with figures,
% and .mar files with information

%% folders:
% - images/elab --> It contains the images resized (? x 255), always with the same number of
% rows (255 pixels)
% - images/elab_cont --> They are the same images as before but binarized. The size is the same,
% maybe with a gap of 1 pixel in the number of columns (width)
% - images/hippocampi --> This is the OUTPUT folder. Here are stored a file with the coordinates of
% the models, and images associated with CA1, CA3, DG and three aproximations to the hippocampus (each one closer than the previous one)
% - images/reference --> reference images resized in format png (the references should be previously preprocessed, the
% system doesn't do it)
% - images/reference_cont --> They are the external contours of the reference brains in format png and resized. They have to be there,
% the system doesn't binarize it automatically. If you want to try with the reference 40 and is not there, that will provoke an error.
% - images/source --> target images (we want to localize the hippocampus within them) in JPG format with original size.
%%

close all

%% IterOtsu SEGMENTATION METHOD PARAMETERS
area_to_select_ca_roi = 1.0; %how much area I want to select / 1.6 - 0.75
area_to_select_dg_roi = 1.0;
radius_factor_ca_roi = 0.75; % if smaller --> bigger circles in the links
radius_factor_dg_roi = 1.0;

%% to discard slices with very bad global registration (before 1.3): HIGHER->bigger
%% differences between the best and the rest. If it is 5 that means that
%% those values 5 times worst than the best one will not be taken into
%% consideration
WORST_GLOBAL_FITNESS_FACTOR = 10;

%% Maximum number of models to try during localization
N_MAX_TEST_MODEL = numberModelsToTest; %don't even try with very bad models!

%% penalty factors for the internal forces of DG [27 0.055 -0.07]
PENALTIES_DG = [27 0.055 -0.3];
% PENALTIES_DG are the values of the contraction factor: first (epsilon p),
% second (epsilon theta) and the third (epsilon c).
%% penalty factors for the internal forces of CA
PENALTIES_CA = [20 0.025 0.065];%[20 0.025 0.1];

%% dimension of the biggest reference image
BIGGEST_REF_IMAGE_SIZE = [255 516]; % [255 517] [255 516]

%% POSSIBLE REFERENCES IN THE ATLAS [ 24; 26; 28; 30; 32; 34; 36; 38; 40; 42; 44; 46; 50 ]
REFERENCES = [9; 10; 11; 12; 13; 14; 15; 16; 17; 18;];

%% ROIs
% these rois has been selected manually (xmin ymin xmax ymax): the first
% two coordinates correspond to the upper left corner of a rectangle which
% includes the area of the hippocampus. The last two coordinates are the
% lower right corner of such a rectangle (asteriscs in the previous image).
%       * -----------
%       |           |
%       |           |
%        -----------*
% These values are calculated over the images in .\images\reference (where are the
% references in low resolution)
HIPPO_REGION.('reference8') = [166 58 245 112]; % 24
HIPPO_REGION.('reference9') = [155 49 240 105]; % 26
HIPPO_REGION.('reference10') = [195 50 273 110]; % 28
HIPPO_REGION.('reference11') = [200 54 272 105]; % 30
HIPPO_REGION.('reference12') = [206 53 272 105]; % 32
HIPPO_REGION.('reference13') = [220 45 299 104]; % 34
HIPPO_REGION.('reference14') = [215 53 278 109]; % 36
HIPPO_REGION.('reference15') = [220 61 289 110]; % 38
HIPPO_REGION.('reference16') = [212 66 271 110]; % 40
HIPPO_REGION.('reference17') = [215 76 278 118]; % 42
HIPPO_REGION.('reference18') = [185 61 258 100];%[224 68 289 114]; % 44
HIPPO_REGION.('reference19') = [180 76 231 107]; % 46
HIPPO_REGION.('reference20') = [200 75 251 106]; % 50

MY_OFFSET = [-5 -5 5 5]; %xmin, ymin, xmax, ymax: increase ROI area
type='sd'; % Type of registration error used see image_difference.m
individualResults.evaluatedRegionsIOECL=0;

[tout1, tout2, tout3, tout4, tout5, tout6, tout7, tout8, tout9, tout10, tout11, tout12, tout13] = deal(0.0);

%output handling
if show_output > -1
    timestamp = datestr(now, 'yymmdd_HHMMSS');
    out_dir_name = sprintf('%s/%s_%s',OUTPUT_FOLDER, image_name,timestamp);
    mkdir(out_dir_name);
    out_file_name = sprintf('%s/%s_%s.txt',out_dir_name,image_name,timestamp);
    f = fopen(out_file_name,'w');
end

%% DEFORMABLE MODEL INITIALIZATION: READING OF IMAGES, EXTRACTION OF
%% EXTERNAL CONTOURS, GLOBAL REGISTRATION, LOCAL REGISTRATION AND BEST
%% REFERENCES SLICE SELECTION
% ROI SIZE
if strcmp(ImageModality,'ISH')
    image_side = 800;
    if referenceToUse == 0
        [tout1, tout2, tout3, tout4, tout5, hip_img, actual_reference, individualResults.bestReferenceSlice, best_fitn, best_ref, fitn] ...
            = deformableModelInitialization(SOURCE_FOLDER,image_name,show_output,imageMatrix, ...
            extract_contours,ELAB_CONT_FOLDER,ELAB_FOLDER,BIGGEST_REF_IMAGE_SIZE, WORST_GLOBAL_FITNESS_FACTOR, REFERENCES, ...
            REF_CONT_FOLDER, type, HIPPO_REGION, MY_OFFSET, REF_FOLDER, f, out_dir_name, timestamp, registration_type);
        
        tic
        %in hip_img there is the extracted ROI
        [roi_img,hip_img] = roiElaboration(hip_img,timestamp,image_name,out_dir_name,show_output, image_side, typeOfROI);
    else % if referencesToUse ~= 0
        tic
        [order best_ref] = sort(REFERENCES==referenceToUse, 'descend');
        best_ref = best_ref(1);
        hip_image_name = sprintf('%s/%s_ROI.png',SOURCE_FOLDER,image_name);
        if ~exist(hip_image_name,'file')% == false
            error(strcat('Source file (',image_name,'.jpg) is not in the correct folder (', SOURCE_FOLDER ,'). Elaboration stopped.'));
        end
        if show_output > 0
            fprintf(1,'Reading Input Image...\n');
        end
        if imageMatrix == 0
            try
                hip_img = cvReadImage(hip_image_name,0);%rgb2gray(imread(source_image_name));
            catch
                hip_img = imread(hip_image_name);
                disp('cvReadImage not available');
            end;
        else
            hip_img = imageMatrix;
        end;
        tout1 = toc;
        tic
        %in hip_img there is the extracted ROI
        [roi_img,hip_img] = roiElaboration(hip_img,timestamp,image_name,out_dir_name,show_output, image_side,typeOfROI);
    end
    tout6 = toc; %time: preprocessing roi
    
    %% PRELIMINAR PHASES OF HIPPOCAMPUS LOCALIZATION
    if sum(ismember(segmentationMethod,'IO')) | sum(ismember(segmentationMethod,'IOECL')) | ...
            sum(ismember(segmentationMethod,'LS') | sum(ismember(segmentationMethod,'GACseeds')) | sum(ismember(segmentationMethod,'CVseeds')))
        % We are checking if the methods used really need localization (IO,
        % IOECL, LS)
        
        output_hippo_bound = 30; % when you find a model, how much space leave around in the saved image
        ERROR_DG = 6.0; %threshold of acceptable error on DG model (1.0)
        ERROR_CA = 12.0; %threshold of acceptable error on CA model (2.0)
        
        if referenceToUse == 0;
            LOCAL_SEARCH_SIZE = 5; %size of the neighborood where to do the local search
            n_m = 1; %index of fitn_sort
            if show_output > 0
                fprintf(1,'Hippocampus Localization starting...\n');
            end
            % se = strel('ball',2,0,0);
            % target = imopen(roi_img,se);
            target = roi_img;
            target([1 size(target,1)],:) = 0;%because i don't want the model to go to the bounds
            target(:,[1 size(target,2)]) = 0;
            [xxx fitn_sort] = sort(fitn); %fitn_sort will contain the order of the models to be used
            model_used = REFERENCES(fitn_sort(n_m));
        else
            N_MAX_TEST_MODEL = numberModelsToTest; % we try the localization with the model proposed, the previous and the next one (or more).
            LOCAL_SEARCH_SIZE = 5; %size of the neighborood where to do the local search
            if show_output > 0
                fprintf(1,'Hippocampus Localization starting...\n');
            end
            % se = strel('ball',2,0,0);
            % target = imopen(roi_img,se);
            target = roi_img;
            target([1 size(target,1)],:) = 0;%because i don't want the model to go to the bounds
            target(:,[1 size(target,2)]) = 0;
            fitn_sort = 1; %fitn_sort will contain the order of the models to be used
            model_used = referenceToUse;
        end;
        
        switch modelsToUse
            case 'classical'
                [movements_min] = get_modelsOld(model_used,1);
            case 'morePoints'
                [movements_min] = get_models_new(model_used,1);
            case 'ASC' % larger points only over 15 images -> more specific models fitted to these images
                [movements_min] = get_synth_models(image_name,1,model_used);
        end
        n_points_dg = size(movements_min,2)/2;
        
        %% PROPER LOCALIZATION
        individualResults.localizationDG.fitnessDE=[];
        individualResults.localizationDG.fitnessPSO=[];
        individualResults.localizationDG.fitnessSS=[];
        individualResults.localizationDG.fitnessSA=[];
        individualResults.localizationDG.fitnessGA=[];
        individualResults.localizationCA.fitnessDE=[];
        individualResults.localizationCA.fitnessPSO=[];
        individualResults.localizationCA.fitnessSS=[];
        individualResults.localizationCA.fitnessSA=[];
        individualResults.localizationCA.fitnessGA=[];
        
        for index = 1:size(metaheuristic,2)
            [tout7, tout8, modelca, modeldg, modelca_out, modeldg_out, finalModelDG, finalModelCA, individualResults, ...
                roi_points, roi_pointsdg, roi_points_out, roi_pointsdg_out, n_points, n_points_dg] = ...
                structureLocalization(n_points_dg, show_output, N_MAX_TEST_MODEL, ...
                referenceToUse, REFERENCES, modelsToUse, PENALTIES_DG, PENALTIES_CA, image_side, ...
                LOCAL_SEARCH_SIZE, ERROR_DG, ERROR_CA, fitn_sort, target,metaheuristic(index),metaheuristicRuns,individualResults, ...
                out_dir_name, image_name, timestamp, best_ref,hip_img,output_hippo_bound, generations, populationSize);
        end;
        
        tout7 = tout7/(metaheuristicRuns);
        tout8 = tout8/(metaheuristicRuns);
        % this will only save the coordinates obtained with the last method used
        % (so, it's thought to be used with only one metaheuristics, not comparing several of them)
        fprintf(f,'DG model\n');
        for i = 1:n_points_dg
            fprintf(f,'%d %d\n',roi_pointsdg(i,1),roi_pointsdg(i,2));
        end
        fprintf(f,'CA model\n');
        for i = 1:n_points
            fprintf(f,'%d %d\n',roi_points(i,1),roi_points(i,2));
        end
        fprintf(f,'External DG model\n');
        for i = 1:n_points_dg
            fprintf(f,'%d %d\n',roi_pointsdg_out(i,1),roi_pointsdg_out(i,2));
        end
        fprintf(f,'External CA model\n');
        for i = 1:n_points
            fprintf(f,'%d %d\n',roi_points_out(i,1),roi_points_out(i,2));
        end
        fclose(f);
        
    else
        
        switch char(metaheuristic)
            case 'DE'
                individualResults.localizationDG.fitnessDE=0.0;
                individualResults.localizationCA.fitnessDE=0.0;
            case 'PSO'
                individualResults.localizationDG.fitnessPSO=0.0;
                individualResults.localizationCA.fitnessPSO=0.0;
            case 'GA'
                individualResults.localizationDG.fitnessGA=0.0;
                individualResults.localizationCA.fitnessGA=0.0;
            case 'SS'
                individualResults.localizationDG.fitnessSS=0.0;
                individualResults.localizationCA.fitnessSS=0.0;
            case 'SA'
                individualResults.localizationDG.fitnessSA=0.0;
                individualResults.localizationCA.fitnessSA=0.0;
        end;
        
        tout7=0.0;
        tout8=0.0;
        
        finalModelDG = 0;
        finalModelCA = 0;
    end;
    
    % [tout9, roi_points, roi_pointsdg, roi_points_out, roi_pointsdg_out] = ...
    %     outputLocalization(show_output, modelca, modeldg, modelca_out, modeldg_out, ...
    %     output_hippo_bound, referenceToUse, out_dir_name, image_name, REFERENCES, timestamp, ...
    %     hip_img, f,modelsToUse, target, best_ref);
    tout9=0.0; % I am not doing outputLocalization, since it's done within the structureLocalization function
    
    individualResults.localizationTime = tout7 + tout8;
    
    if referenceToUse ~= 0
        tout2 = 0.0;
        tout3 = 0.0;
        tout4 = 0.0;
        tout5 = 0.0;
    end
    
    if referenceToUse ~= 0
        hippo_filename = sprintf('%s/%s_orig_im%d_%s.png',out_dir_name,image_name, REFERENCES(best_ref),timestamp); %save extracted ROI only once
        imwrite(hip_img, hippo_filename,'png');
    end;
else %% IF WE DON'T USE ISH, THAT MEANS THAT WE'RE NOT USING ASM (LOCALIZATION)
    tout1 = 0.0;
    tout2 = 0.0;
    tout3 = 0.0;
    tout4 = 0.0;
    tout5 = 0.0;
    tout6 = 0.0;
    tout7 = 0.0;
    tout8 = 0.0;
    tout9 = 0.0;
    finalModelDG = -1;
    finalModelCA = -1;
    best_ref = -1;
    individualResults.localizationDG.fitnessDE=0.0;
    individualResults.localizationCA.fitnessDE=0.0;
    individualResults.localizationDG.fitnessPSO=0.0;
    individualResults.localizationCA.fitnessPSO=0.0;
    individualResults.localizationDG.fitnessGA=0.0;
    individualResults.localizationCA.fitnessGA=0.0;
    individualResults.localizationDG.fitnessSS=0.0;
    individualResults.localizationCA.fitnessSS=0.0;
    individualResults.localizationDG.fitnessSA=0.0;
    individualResults.localizationCA.fitnessSA=0.0;
    individualResults.localizationTime = 0.0;
    individualResults.bestReferenceSlice = -1;
    hip_img = imageMatrix;
    hippo_filename = sprintf('%s/%s_orig_im_%s.png',out_dir_name,image_name, timestamp); %save extracted ROI only once
    imwrite(hip_img, hippo_filename,'png');
end;

%% SEGMENTATION
if referenceToUse ~= 0
    individualResults.bestReferenceSlice = referenceToUse;
end;

if performSegmentation
    %     im = hip_img;
    %     orig_im = im;
    if strcmp(ImageModality,'MRI')
        minimumSize = 75;
        % MRI: 107 structures to segment. Smallest regions: 1, 6, 10, 6, 8, 3, 1
        % 3, 1, 1, 83, 98, 99. If we select 75 we are able to segment 97 ->
        % 90%
    elseif (strcmp(ImageModality,'Lungs') | strcmp(ImageModality,'Knee'))
        minimumSize = 200;
        % CT: 25 structures to segment. Smallest regions: 1, 67, and 358 pixels.
        % We select 200, so we are able to segment 23 -> 92%
    elseif strcmp(ImageModality,'ISH')
        minimumSize = 25000;
        % Third of the average size of the structures to segment.
        
    end;
    
    if sum(strcmp(segmentationMethod,'IO')) > 0
        %% Segmentation of hippocampus from the located points (using Iterative Otsu's Thresholding Method)
        [tout10, newmask, maskpoly, im, orig_im, im2] = ...
            segmentationIterativeOtsuThresholding(roi_points, roi_pointsdg, roi_points_out, roi_pointsdg_out, hip_img, ...
            area_to_select_ca_roi, area_to_select_dg_roi, radius_factor_ca_roi, SaveSegmentationEvolution, ...
            radius_factor_dg_roi, show_output, out_dir_name,image_name, REFERENCES,best_ref,timestamp);
        % orig_im contains the original ROI (b/w and real size)
        % im contains the ROI including the window masks (the rest is considered background = white)
        % maskpoly contains the window masks
        % newmask is the result
        % im2 contains the original ROI after smoothing and so on
        
        individualResults.IO.segmentationTime = tout10;
        
        if ValidateSegmentationResults == 1
            % VALIDATE RESULTS
            % If there's not a ground truth it will return 0.0 in all parameters
            if (referenceToUse > 0 | referenceToUse == -1)
                useRegistrationToValidateResults = 0;
            else
                useRegistrationToValidateResults = 1;
            end;
            [individualResults.IO.TP individualResults.IO.FP individualResults.IO.FN ...
                individualResults.IO.DSC individualResults.IO.HD individualResults.IO.J] = ...
                validateResults(newmask,image_name,SOURCE_FOLDER,out_dir_name, show_output,'IO', ...
                useRegistrationToValidateResults,MODEL_TO_COMPARE,1);
        end;
        
        % time processing
        tout11 = 0.0;
        individualResults.IO.totalTime = tout1 + tout2 + tout3 + tout4 + tout5 + tout6 + tout7 + tout8 + tout9 + tout10 + tout11;
        if (isunix==0)
            if (writeTimeExcel==2) %append
                writeExcelFile('append','_IOMethod_',FILENAME_TIME, image_name, modelsToUse,...
                    tout1 , tout2 , tout3 , tout4 , tout5 , tout6 , tout7 , tout8 , tout9 , ...
                    tout10, tout11,individualResults.IO.totalTime);
            elseif (writeTimeExcel==1) % we create a new file
                writeExcelFile('create','_IOMethod_',FILENAME_TIME, image_name, modelsToUse,...
                    tout1 , tout2 , tout3 , tout4 , tout5 , tout6 , tout7 , tout8 , tout9 , ...
                    tout10, tout11,individualResults.IO.totalTime);
                writeTimeExcel=2;
            elseif (writeTimeExcel==0)
                %nothing
            end;
        end;
        
        if (refinement)
            individualResults.evaluatedRegionsIOECL = 0;
            %% REFINEMENT USING ENSEMBLE CLASSIFIERS
            [tout11, newmaskExpansion, individualResults] = ...
                expansionWithClassifiers(SaveSegmentationEvolution, maskpoly, newmask, orig_im, ...
                out_dir_name,image_name, REFERENCES,best_ref,timestamp, im2, individualResults);
            
            % time processing
            individualResults.IOECL.totalTime = tout1 + tout2 + tout3 + tout4 + tout5 + tout6 + tout7 + tout8 + tout9 + tout10 + tout11;
            individualResults.IOECL.segmentationTime = tout11;
            
            if ValidateSegmentationResults==1
                %Validate results
                % If there's not a ground truth it will return 0.0 in all
                % parameters
                if (referenceToUse > 0 | referenceToUse == -1)
                    useRegistrationToValidateResults = 0;
                else
                    useRegistrationToValidateResults = 1;
                end;
                [individualResults.IOECL.TP individualResults.IOECL.FP individualResults.IOECL.FN ...
                    individualResults.IOECL.DSC individualResults.IOECL.HD individualResults.IOECL.J] = ...
                    validateResults(newmaskExpansion,image_name,SOURCE_FOLDER, ...
                    out_dir_name, show_output,'IOECL',useRegistrationToValidateResults,MODEL_TO_COMPARE,1);
            end;
            if (isunix==0)
                if (writeTimeExcel==2) %append
                    writeExcelFile('append','_IOECLMethod_',FILENAME_TIME, image_name, modelsToUse,...
                        tout1 , tout2 , tout3 , tout4 , tout5 , tout6 , tout7 , tout8 , tout9 , ...
                        tout10, tout11,individualResults.IOECL.totalTime);
                elseif (writeTimeExcel==1) % we create a new file
                    writeExcelFile('create','_IOECLMethod_',FILENAME_TIME, image_name, modelsToUse,...
                        tout1 , tout2 , tout3 , tout4 , tout5 , tout6 , tout7 , tout8 , tout9 , ...
                        tout10, tout11,individualResults.IOECL.totalTime);
                    writeTimeExcel=2;
                elseif (writeTimeExcel==0)
                    %nothing
                end;
            end;
        end;
        
    end
    
    %% Segmentation of hippocampus - using LevelSets: gradient map,
    %% gaussian smoothing and grey levels (mean and std)
    if (sum(strcmp(segmentationMethod,'LS')) > 0)
        orig_im = hip_img;
        
        [tout12, newmaskLS] = ...
            segmentationSimpleLevelSet(roi_points, roi_pointsdg, orig_im, ...
            SaveSegmentationEvolution, out_dir_name,image_name, REFERENCES,best_ref,timestamp);
        
        % time processing
        individualResults.LS.segmentationTime = tout12;
        individualResults.LS.totalTime = tout1 + tout2 + tout3 + tout4 + tout5 + tout6 + tout7 + tout8 + tout9 + tout12;
        
        if ValidateSegmentationResults==1
            % VALIDATE RESULTS
            % If there's not a ground truth it will return 0.0 in all
            % parameters
            if (referenceToUse > 0 | referenceToUse == -1)
                useRegistrationToValidateResults = 0;
            else
                useRegistrationToValidateResults = 1;
            end;
            [individualResults.LS.TP individualResults.LS.FP individualResults.LS.FN ...
                individualResults.LS.DSC individualResults.LS.HD individualResults.LS.J] = ...
                validateResults(newmaskLS,image_name,SOURCE_FOLDER,out_dir_name, show_output,'LS',...
                useRegistrationToValidateResults,MODEL_TO_COMPARE,1);
        end;
        if (isunix==0)
            if (writeTimeExcel==2) %append
                writeExcelFile('append','_LevelSetsMethod_',FILENAME_TIME, image_name, modelsToUse,...
                    tout1 , tout2 , tout3 , tout4 , tout5 , tout6 , tout7 , tout8 , tout9 , ...
                    tout12, 0.0,individualResults.LS.totalTime);
            elseif (writeTimeExcel==1) % we create a new file
                writeExcelFile('create','_LevelSetsMethod_',FILENAME_TIME, image_name, modelsToUse,...
                    tout1 , tout2 , tout3 , tout4 , tout5 , tout6 , tout7 , tout8 , tout9 , ...
                    tout12, 0.0,individualResults.LS.totalTime);
                writeTimeExcel=2;
            elseif (writeTimeExcel==0)
                %nothing
            end;
        end;
    end;
    
    %% Segmentation of hippocampus - using Fuzzy C-Means
    if (sum(strcmp(segmentationMethod,'FCM')) > 0)
        orig_im = hip_img;
        
        [tout13, newmaskFCM] = ...
            segmentationFCM(orig_im, out_dir_name,image_name, REFERENCES,best_ref,timestamp);
        
        % time processing
        individualResults.FCM.segmentationTime = tout13;
        individualResults.FCM.totalTime = tout1 + tout2 + tout3 + tout4 + tout5 + tout13;
        
        if ValidateSegmentationResults==1
            % VALIDATE RESULTS
            % If there's not a ground truth it will return 0.0 in all
            % parameters
            if (referenceToUse > 0 | referenceToUse == -1)
                useRegistrationToValidateResults = 0;
            else
                useRegistrationToValidateResults = 1;
            end;
            [individualResults.FCM.TP individualResults.FCM.FP individualResults.FCM.FN ...
                individualResults.FCM.DSC individualResults.FCM.HD individualResults.FCM.J] = ...
                validateResults(newmaskFCM,image_name,SOURCE_FOLDER,out_dir_name, show_output,'FCM',...
                useRegistrationToValidateResults,MODEL_TO_COMPARE,1);
        end;
        if (isunix==0)
            if (writeTimeExcel==2) %append
                writeExcelFile('append','_FCMMethod_',FILENAME_TIME, image_name, modelsToUse,...
                    tout1 , tout2 , tout3 , tout4 , tout5 , 0.0 , 0.0 , 0.0 , 0.0 , ...
                    tout13, 0.0,individualResults.FCM.totalTime);
            elseif (writeTimeExcel==1) % we create a new file
                writeExcelFile('create','_FCMMethod_',FILENAME_TIME, image_name, modelsToUse,...
                    tout1 , tout2 , tout3 , tout4 , tout5 , 0.0 , 0.0 , 0.0 , 0.0 , ...
                    tout13, 0.0,individualResults.FCM.totalTime);
                writeTimeExcel=2;
            elseif (writeTimeExcel==0)
                %nothing
            end;
        end;
    end;
    
    
    %% Segmentation of hippocampus - using Soft Thresholding
    if (sum(strcmp(segmentationMethod,'ST')) > 0)
        orig_im = hip_img;
        %         if strcmp(ImageModality,'MRI')
        %             minimumSize = 100;
        %         elseif strcmp(ImageModality,'CT')
        %             minimumSize = 200;
        %         elseif strcmp(ImageModality,'ISH')
        %             minimumSize = 5000; % if 1000x1000 -> 25000
        %         end;
        [tout14, newmaskST] = ...
            segmentationST(orig_im, out_dir_name,image_name, SOURCE_FOLDER,REFERENCES,best_ref,timestamp,minimumSize);
        
        % time processing
        individualResults.ST.segmentationTime = tout14;
        individualResults.ST.totalTime = tout1 + tout2 + tout3 + tout4 + tout5 + tout14;
        
        if ValidateSegmentationResults==1
            % VALIDATE RESULTS
            % If there's not a ground truth it will return 0.0 in all
            % parameters
            if (referenceToUse > 0 | referenceToUse == -1)
                useRegistrationToValidateResults = 0;
            else
                useRegistrationToValidateResults = 1;
            end;
            [individualResults.ST.TP individualResults.ST.FP individualResults.ST.FN ...
                individualResults.ST.DSC individualResults.ST.HD individualResults.ST.J] = ...
                validateResults(newmaskST,image_name,SOURCE_FOLDER,out_dir_name, show_output,'ST',...
                useRegistrationToValidateResults,MODEL_TO_COMPARE,1);
        end;
        
        if (isunix==0)
            if (writeTimeExcel==2) %append
                writeExcelFile('append','_STMethod_',FILENAME_TIME, image_name, modelsToUse,...
                    tout1 , tout2 , tout3 , tout4 , tout5 , 0.0 , 0.0 , 0.0 , 0.0 , ...
                    tout14, 0.0,individualResults.ST.totalTime);
            elseif (writeTimeExcel==1) % we create a new file
                writeExcelFile('create','_STMethod_',FILENAME_TIME, image_name, modelsToUse,...
                    tout1 , tout2 , tout3 , tout4 , tout5 , 0.0 , 0.0 , 0.0 , 0.0 , ...
                    tout14, 0.0,individualResults.ST.totalTime);
                writeTimeExcel=2;
            elseif (writeTimeExcel==0)
                %nothing
            end;
        end;
    end;
    
    
    %% Segmentation of hippocampus - using Otsu Thresholding Method
    if (sum(strcmp(segmentationMethod,'Otsu')) > 0)
        orig_im = hip_img;
        
        [tout15, newmaskOtsu] = ...
            segmentationOtsu(orig_im, out_dir_name,image_name, REFERENCES,best_ref,timestamp);
        
        % time processing
        individualResults.Otsu.segmentationTime = tout15;
        individualResults.Otsu.totalTime = tout1 + tout2 + tout3 + tout4 + tout5 + tout15;
        
        if ValidateSegmentationResults==1
            % VALIDATE RESULTS
            % If there's not a ground truth it will return 0.0 in all
            % parameters
            if (referenceToUse > 0 | referenceToUse == -1)
                useRegistrationToValidateResults = 0;
            else
                useRegistrationToValidateResults = 1;
            end;
            [individualResults.Otsu.TP individualResults.Otsu.FP individualResults.Otsu.FN ...
                individualResults.Otsu.DSC individualResults.Otsu.HD individualResults.Otsu.J] = ...
                validateResults(newmaskOtsu,image_name,SOURCE_FOLDER,out_dir_name, show_output,'Otsu',...
                useRegistrationToValidateResults,MODEL_TO_COMPARE,1);
        end;
        
        if (isunix==0)
            if (writeTimeExcel==2) %append
                writeExcelFile('append','_OtsuMethod_',FILENAME_TIME, image_name, modelsToUse,...
                    tout1 , tout2 , tout3 , tout4 , tout5 , 0.0 , 0.0 , 0.0 , 0.0 , ...
                    tout15, 0.0,individualResults.Otsu.totalTime);
            elseif (writeTimeExcel==1) % we create a new file
                writeExcelFile('create','_OtsuMethod_',FILENAME_TIME, image_name, modelsToUse,...
                    tout1 , tout2 , tout3 , tout4 , tout5 , 0.0 , 0.0 , 0.0 , 0.0 , ...
                    tout15, 0.0,individualResults.Otsu.totalTime);
                writeTimeExcel=2;
            elseif (writeTimeExcel==0)
                %nothing
            end;
        end;
    end;
    
    %% Segmentation of hippocampus - using K-Means
    if (sum(strcmp(segmentationMethod,'KM')) > 0)
        orig_im = hip_img;
        
        [tout19, newmaskKM] = ...
            segmentationKM(orig_im, out_dir_name,image_name, REFERENCES,best_ref,timestamp);
        
        % time processing
        individualResults.KM.segmentationTime = tout19;
        individualResults.KM.totalTime = tout1 + tout2 + tout3 + tout4 + tout5 + tout19;
        
        if ValidateSegmentationResults==1
            % VALIDATE RESULTS
            % If there's not a ground truth it will return 0.0 in all
            % parameters
            if (referenceToUse > 0 | referenceToUse == -1)
                useRegistrationToValidateResults = 0;
            else
                useRegistrationToValidateResults = 1;
            end;
            [individualResults.KM.TP individualResults.KM.FP individualResults.KM.FN ...
                individualResults.KM.DSC individualResults.KM.HD individualResults.KM.J] = ...
                validateResults(newmaskKM,image_name,SOURCE_FOLDER,out_dir_name, show_output,'KM',...
                useRegistrationToValidateResults,MODEL_TO_COMPARE,1);
        end;
        
        if (isunix==0)
            if (writeTimeExcel==2) %append
                writeExcelFile('append','_KMMethod_',FILENAME_TIME, image_name, modelsToUse,...
                    tout1 , tout2 , tout3 , tout4 , tout5 , 0.0 , 0.0 , 0.0 , 0.0 , ...
                    tout19, 0.0,individualResults.KM.totalTime);
            elseif (writeTimeExcel==1) % we create a new file
                writeExcelFile('create','_KMMethod_',FILENAME_TIME, image_name, modelsToUse,...
                    tout1 , tout2 , tout3 , tout4 , tout5 , 0.0 , 0.0 , 0.0 , 0.0 , ...
                    tout19, 0.0,individualResults.KM.totalTime);
                writeTimeExcel=2;
            elseif (writeTimeExcel==0)
                %nothing
            end;
        end;
    end;
    
    %% Segmentation of hippocampus - using Affine Registration (Elastix)
    phase1 = 0.0;
    if (sum(strcmp(segmentationMethod,'AS')) > 0)
        orig_im = hip_img;
        
        [tout16, phase1, newmaskAS] = ...
            segmentationAS(orig_im, out_dir_name,image_name, REFERENCES,best_ref,timestamp, ...
            SOURCE_FOLDER, MODEL_TO_COMPARE,ImageModality);
        
        % time processing
        individualResults.AS.segmentationTime = tout16;
        individualResults.AS.totalTime = tout16 + tout1 + tout2 + tout3 + tout4 + tout5;
        
        if ValidateSegmentationResults==1
            % VALIDATE RESULTS
            % If there's not a ground truth it will return 0.0 in all
            % parameters
            if (referenceToUse > 0 | referenceToUse == -1)
                useRegistrationToValidateResults = 0;
            else
                useRegistrationToValidateResults = 1;
            end;
            [individualResults.AS.TP individualResults.AS.FP individualResults.AS.FN ...
                individualResults.AS.DSC individualResults.AS.HD individualResults.AS.J] = ...
                validateResults(newmaskAS,image_name,SOURCE_FOLDER,out_dir_name, show_output,'AS',...
                useRegistrationToValidateResults,MODEL_TO_COMPARE,1);
        end;
        
        if (isunix==0)
            if (writeTimeExcel==2) %append
                writeExcelFile('append','_AffineSegmentationMethod_',FILENAME_TIME, image_name, modelsToUse,...
                    tout1 , tout2 , tout3 , tout4 , tout5 , 0.0 , 0.0 , 0.0 , 0.0 , ...
                    tout16, 0.0,individualResults.AS.totalTime);
            elseif (writeTimeExcel==1) % we create a new file
                writeExcelFile('create','_AffineSegmentationMethod_',FILENAME_TIME, image_name, modelsToUse,...
                    tout1 , tout2 , tout3 , tout4 , tout5 , 0.0 , 0.0 , 0.0 , 0.0 , ...
                    tout16, 0.0,individualResults.AS.totalTime);
                writeTimeExcel=2;
            elseif (writeTimeExcel==0)
                %nothing
            end;
        end;
    end;
    
    %% Segmentation of hippocampus - using Deformable Registration
    %% (B-splines)
    if (sum(strcmp(segmentationMethod,'DS')) > 0)
        orig_im = hip_img;
        
        [tout20, newmaskDS] = ...
            segmentationDS(orig_im, out_dir_name,image_name, REFERENCES,best_ref,timestamp, phase1,ImageModality);
        
        % time processing
        individualResults.DS.segmentationTime = tout20;
        individualResults.DS.totalTime = tout20 + tout1 + tout2 + tout3 + tout4 + tout5;
        
        if ValidateSegmentationResults==1
            % VALIDATE RESULTS
            % If there's not a ground truth it will return 0.0 in all
            % parameters
            if (referenceToUse > 0 | referenceToUse == -1)
                useRegistrationToValidateResults = 0;
            else
                useRegistrationToValidateResults = 1;
            end;
            [individualResults.DS.TP individualResults.DS.FP individualResults.DS.FN ...
                individualResults.DS.DSC individualResults.DS.HD individualResults.DS.J] = ...
                validateResults(newmaskDS,image_name,SOURCE_FOLDER,out_dir_name, show_output,'DS',...
                useRegistrationToValidateResults,MODEL_TO_COMPARE,1);
        end;
        
        if (isunix==0)
            if (writeTimeExcel==2) %append
                writeExcelFile('append','_DeformableSegmentationMethod_',FILENAME_TIME, image_name, modelsToUse,...
                    tout1 , tout2 , tout3 , tout4 , tout5 , 0.0 , 0.0 , 0.0 , 0.0 , ...
                    tout20, 0.0,individualResults.DS.totalTime);
            elseif (writeTimeExcel==1) % we create a new file
                writeExcelFile('create','_DeformableSegmentationMethod_',FILENAME_TIME, image_name, modelsToUse,...
                    tout1 , tout2 , tout3 , tout4 , tout5 , 0.0 , 0.0 , 0.0 , 0.0 , ...
                    tout20, 0.0,individualResults.DS.totalTime);
                writeTimeExcel=2;
            elseif (writeTimeExcel==0)
                %nothing
            end;
        end;
    end;
    
    %% Segmentation of hippocampus - using Geodesic Active Contours
    if (sum(strcmp(segmentationMethod,'GAC') | sum(ismember(segmentationMethod,'GACseeds')) | sum(ismember(segmentationMethod,'DSGAC'))) > 0)
        orig_im = hip_img;
        if strcmp(ImageModality,'ISH')
            countour_weight = 3; %3
            expansion_weight = -1;%-1
            medianFilterSize = 10;%10
            %             minimumSize = 5000; % if 1000x1000 -> 25000
        elseif strcmp(ImageModality,'Lungs')
            %             if image_name(1) == 'l' %Lungs
            countour_weight = 2; %3
            expansion_weight = -1;%-1
            medianFilterSize = 3;%10
            %             elseif image_name(1) == 'r' %Knee
        elseif strcmp(ImageModality,'Knee')
            countour_weight = 1; %3
            expansion_weight = -0.5;%-1
            medianFilterSize = 1;%10
            %         end;
            %             minimumSize = 200;
        elseif strcmp(ImageModality,'MRI')
            countour_weight = 3; %3
            expansion_weight = -1;%-1
            medianFilterSize = 3;%10
            %             minimumSize = 100; % WE DON'T REMOVE ANYTHING
        end;
        %% SEEDED GAC
        if sum(ismember(segmentationMethod,'GACseeds')) % seeds from localization with DE
            [tout17, newmaskGAC] = ...
                segmentationGAC(orig_im, out_dir_name,image_name, REFERENCES,...
                best_ref,timestamp,maskpoly, countour_weight, expansion_weight, medianFilterSize,minimumSize,'GACseeds',ImageModality);
            % time processing
            individualResults.GACseeds.segmentationTime = tout17+tout7 + tout8;
            individualResults.GACseeds.totalTime = tout17 + tout1 + tout2 + tout3 + tout4 + tout5 + tout6 + tout7 + tout8 + tout10;
            if ValidateSegmentationResults==1
                if (referenceToUse > 0 | referenceToUse == -1)
                    useRegistrationToValidateResults = 0;
                else
                    useRegistrationToValidateResults = 1;
                end;
                [individualResults.GACseeds.TP individualResults.GACseeds.FP individualResults.GACseeds.FN ...
                    individualResults.GACseeds.DSC individualResults.GACseeds.HD individualResults.GACseeds.J] = ...
                    validateResults(newmaskGAC,image_name,SOURCE_FOLDER,out_dir_name, show_output,'GACseeds',...
                    useRegistrationToValidateResults,MODEL_TO_COMPARE,1);
            end;
            
            if (isunix==0)
                if (writeTimeExcel==2) %append
                    writeExcelFile('append','_GACseedsMethod_',FILENAME_TIME, image_name, modelsToUse,...
                        tout1 , tout2 , tout3 , tout4 , tout5 , tout6 , tout7 , tout8 , 0.0 , ...
                        tout17+tout10, 0.0,individualResults.GACseeds.totalTime);
                elseif (writeTimeExcel==1) % we create a new file
                    writeExcelFile('create','_GACseedsMethod_',FILENAME_TIME, image_name, modelsToUse,...
                        tout1 , tout2 , tout3 , tout4 , tout5 , tout6 , tout7 , tout8 , 0.0 , ...
                        tout17+tout10, 0.0,individualResults.GACseeds.totalTime);
                    writeTimeExcel=2;
                elseif (writeTimeExcel==0)
                    %nothing
                end;
            end;
            
        end;
        if (sum(strcmp(segmentationMethod,'DSGAC')) > 0)
            %% Segmentation of hippocampus - using Deformable Registration (B-splines)
            %% and Level Set Method (in this case, Geodesic Active Contours)
            if strcmp(ImageModality,'MRI')
                countour_weight = 3; %3
                expansion_weight = -0.5;%-1
                medianFilterSize = 1;%10
            end;
            if strcmp(ImageModality,'ISH')
                countour_weight = 3; %3
                expansion_weight = -1;%-1
                medianFilterSize = 10;%10
            end;
            if strcmp(ImageModality,'Lungs')
                %                 if image_name(1) == 'l' %Lungs
                countour_weight = 3; %3
                expansion_weight = -1;%-1
                medianFilterSize = 3;%10
            elseif strcmp(ImageModality,'Knee')
                %                 elseif image_name(1) == 'r' %Knee
                countour_weight = 3; %3
                expansion_weight = -0.5;%-1
                medianFilterSize = 1;%10
                %             end;
            end;
            orig_im = hip_img;
            
            [tout21, newmaskDSGAC] = ...
                segmentationGAC(orig_im, out_dir_name,image_name, REFERENCES,...
                best_ref,timestamp,newmaskDS, countour_weight, expansion_weight, medianFilterSize,minimumSize,'DSGAC',ImageModality);
            
            % time processing
            individualResults.DSGAC.segmentationTime = tout21 + tout20;
            individualResults.DSGAC.totalTime = tout21 + tout20 + tout1 + tout2 + tout3 + tout4 + tout5;
            
            if ValidateSegmentationResults==1
                % VALIDATE RESULTS
                % If there's not a ground truth it will return 0.0 in all
                % parameters
                if (referenceToUse > 0 | referenceToUse == -1)
                    useRegistrationToValidateResults = 0;
                else
                    useRegistrationToValidateResults = 1;
                end;
                [individualResults.DSGAC.TP individualResults.DSGAC.FP individualResults.DSGAC.FN ...
                    individualResults.DSGAC.DSC individualResults.DSGAC.HD individualResults.DSGAC.J] = ...
                    validateResults(newmaskDSGAC,image_name,SOURCE_FOLDER,out_dir_name, show_output,'DSGAC',...
                    useRegistrationToValidateResults,MODEL_TO_COMPARE,1);
            end;
            
            if (isunix==0)
                if (writeTimeExcel==2) %append
                    writeExcelFile('append','_DSGACMethod_',FILENAME_TIME, image_name, modelsToUse,...
                        tout1 , tout2 , tout3 , tout4 , tout5 , 0.0 , 0.0 , 0.0 , 0.0 , ...
                        tout20, 0.0,individualResults.DSGAC.totalTime);
                elseif (writeTimeExcel==1) % we create a new file
                    writeExcelFile('create','_DSGACMethod_',FILENAME_TIME, image_name, modelsToUse,...
                        tout1 , tout2 , tout3 , tout4 , tout5 , 0.0 , 0.0 , 0.0 , 0.0 , ...
                        tout20, 0.0,individualResults.DSGAC.totalTime);
                    writeTimeExcel=2;
                elseif (writeTimeExcel==0)
                    %nothing
                end;
            end;
            %% NOT SEEDED GAC
        end;
        if (sum(strcmp(segmentationMethod,'GAC')) > 0)
            if strcmp(ImageModality,'ISH')
                countour_weight = 3; %3
                expansion_weight = -1;%-1
                medianFilterSize = 10;%10
                %             minimumSize = 5000; % if 1000x1000 -> 25000
            elseif strcmp(ImageModality,'Lungs')
                %                 if image_name(1) == 'l' %Lungs
                countour_weight = 2; %3
                expansion_weight = -1;%-1
                medianFilterSize = 3;%10
            elseif strcmp(ImageModality,'Knee')
                %                 elseif image_name(1) == 'r' %Knee
                countour_weight = 1; %3
                expansion_weight = -0.5;%-1
                medianFilterSize = 1;%10
                %         end;
                %             minimumSize = 200;
            elseif strcmp(ImageModality,'MRI')
                countour_weight = 3; %3
                expansion_weight = -1;%-1
                medianFilterSize = 3;%10
                %             minimumSize = 100; % WE DON'T REMOVE ANYTHING
            end;
            [tout17, newmaskGAC] = ...
                segmentationGAC(orig_im, out_dir_name,image_name, REFERENCES,...
                best_ref,timestamp,0, countour_weight, expansion_weight, medianFilterSize,minimumSize,'GAC',ImageModality);
            % time processing
            individualResults.GAC.segmentationTime = tout17;
            individualResults.GAC.totalTime = tout17 + tout1 + tout2 + tout3 + tout4 + tout5;
            if ValidateSegmentationResults==1
                if (referenceToUse > 0 | referenceToUse == -1)
                    useRegistrationToValidateResults = 0;
                else
                    useRegistrationToValidateResults = 1;
                end;
                [individualResults.GAC.TP individualResults.GAC.FP individualResults.GAC.FN ...
                    individualResults.GAC.DSC individualResults.GAC.HD individualResults.GAC.J] = ...
                    validateResults(newmaskGAC,image_name,SOURCE_FOLDER,out_dir_name, show_output,'GAC',...
                    useRegistrationToValidateResults,MODEL_TO_COMPARE,1);
            end;
            
            if (isunix==0)
                if (writeTimeExcel==2) %append
                    writeExcelFile('append','_GACMethod_',FILENAME_TIME, image_name, modelsToUse,...
                        tout1 , tout2 , tout3 , tout4 , tout5 , 0.0 , 0.0 , 0.0 , 0.0 , ...
                        tout17, 0.0,individualResults.GAC.totalTime);
                elseif (writeTimeExcel==1) % we create a new file
                    writeExcelFile('create','_GACMethod_',FILENAME_TIME, image_name, modelsToUse,...
                        tout1 , tout2 , tout3 , tout4 , tout5 , 0.0 , 0.0 , 0.0 , 0.0 , ...
                        tout17, 0.0,individualResults.GAC.totalTime);
                    writeTimeExcel=2;
                elseif (writeTimeExcel==0)
                    %nothing
                end;
            end;
        end;
    end;
    
    %% Segmentation of hippocampus - using Chan & Vese model. It
    %% implements: "Active Contours Without Edges" by Chan and Vese
    if (sum(strcmp(segmentationMethod,'CV') | sum(ismember(segmentationMethod,'CVseeds')) | sum(ismember(segmentationMethod,'DSCV'))) > 0)
        orig_im = hip_img;
        if strcmp(ImageModality,'ISH')
            weightLengthTerm = 0.01;
            sizeMedFilt = 5;
            lambda1 = 1;
            lambda2 = 1;
            %             minimumSize = 5000; % if 1000x1000 -> 25000
        elseif strcmp(ImageModality,'Lungs')
            %             if image_name(1) == 'l' %Lungs
            weightLengthTerm = 0.01;
            sizeMedFilt = 3;
            lambda1 = 5;
            lambda2 = 2;
        elseif strcmp(ImageModality,'Knee')
            %             elseif image_name(1) == 'r' %Knee
            weightLengthTerm = 0.01;
            sizeMedFilt = 3;
            lambda1 = 5;
            lambda2 = 2;
            %     end;
            %             minimumSize = 200;
        elseif strcmp(ImageModality,'MRI')
            weightLengthTerm = 0.01;
            sizeMedFilt = 1;
            lambda1 = 1;
            lambda2 = 1;
            %             minimumSize = 100; % WE DON'T REMOVE ANYTHING
        end;
        %% SEEDED CV
        if sum(ismember(segmentationMethod,'CVseeds')) % seeds from localization with DE
            [tout18, newmaskCV] = ...
                segmentationCV(orig_im, out_dir_name,image_name, SOURCE_FOLDER,...
                REFERENCES,best_ref,timestamp,maskpoly, weightLengthTerm, sizeMedFilt,lambda1,lambda2,minimumSize,'CVseeds');
            
            % time processing
            individualResults.CVseeds.segmentationTime = tout18;
            individualResults.CVseeds.totalTime = tout18 + tout1 + tout2 + tout3 + tout4 + tout5 + tout6 + tout7 + tout8 + tout10;
            
            if ValidateSegmentationResults==1
                if (referenceToUse > 0 | referenceToUse == -1)
                    useRegistrationToValidateResults = 0;
                else
                    useRegistrationToValidateResults = 1;
                end;
                [individualResults.CVseeds.TP individualResults.CVseeds.FP individualResults.CVseeds.FN ...
                    individualResults.CVseeds.DSC individualResults.CVseeds.HD individualResults.CVseeds.J] = ...
                    validateResults(newmaskCV,image_name,SOURCE_FOLDER,out_dir_name, show_output,'CVseeds',...
                    useRegistrationToValidateResults,MODEL_TO_COMPARE,1);
            end;
            
            if (isunix==0)
                if (writeTimeExcel==2) %append
                    writeExcelFile('append','_CVseedsMethod_',FILENAME_TIME, image_name, modelsToUse,...
                        tout1 , tout2 , tout3 , tout4 , tout5 , tout6 , tout7 , tout8 , 0.0 , ...
                        tout18 + tout10, 0.0,individualResults.CVseeds.totalTime);
                elseif (writeTimeExcel==1) % we create a new file
                    writeExcelFile('create','_CVseedsMethod_',FILENAME_TIME, image_name, modelsToUse,...
                        tout1 , tout2 , tout3 , tout4 , tout5 , tout6 , tout7 , tout8 , 0.0 , ...
                        tout18 + tout10, 0.0,individualResults.CVseeds.totalTime);
                    writeTimeExcel=2;
                elseif (writeTimeExcel==0)
                    %nothing
                end;
            end;
            %% Segmentation of hippocampus - using Deformable Registration (B-splines)
            %% and Level Set Method (in this case, Chan & Vese model)
        end;
        if (sum(strcmp(segmentationMethod,'DSCV')) > 0)
            orig_im = hip_img;
            if strcmp(ImageModality,'MRI') | strcmp(ImageModality,'ISH')
                weightLengthTerm = 0.01;
                sizeMedFilt = 5;
                lambda1 = 1;
                lambda2 = 1;
            end;
            if strcmp(ImageModality,'Lungs')
                %                 if image_name(1) == 'l' %Lungs
                weightLengthTerm = 0.01;
                sizeMedFilt = 3;
                lambda1 = 1;
                lambda2 = 5;
            elseif strcmp(ImageModality,'Knee')
                %                 elseif image_name(1) == 'r' %Knee
                weightLengthTerm = 0.01;
                sizeMedFilt = 1;
                lambda1 = 1;
                lambda2 = 1;
                %     end;
            end;
            
            [tout22, newmaskDSCV] = ...
                segmentationCV(orig_im, out_dir_name,image_name, SOURCE_FOLDER,...
                REFERENCES,best_ref,timestamp,newmaskDS, weightLengthTerm, sizeMedFilt,lambda1,lambda2,minimumSize,'DSCV');
            
            % time processing
            individualResults.DSCV.segmentationTime = tout22 + tout20;
            individualResults.DSCV.totalTime = tout22 + tout20 + tout1 + tout2 + tout3 + tout4 + tout5;
            
            if ValidateSegmentationResults==1
                % VALIDATE RESULTS
                % If there's not a ground truth it will return 0.0 in all
                % parameters
                if (referenceToUse > 0 | referenceToUse == -1)
                    useRegistrationToValidateResults = 0;
                else
                    useRegistrationToValidateResults = 1;
                end;
                [individualResults.DSCV.TP individualResults.DSCV.FP individualResults.DSCV.FN ...
                    individualResults.DSCV.DSC individualResults.DSCV.HD individualResults.DSCV.J] = ...
                    validateResults(newmaskDSCV,image_name,SOURCE_FOLDER,out_dir_name, show_output,'DSCV',...
                    useRegistrationToValidateResults,MODEL_TO_COMPARE,1);
            end;
            
            if (isunix==0)
                if (writeTimeExcel==2) %append
                    writeExcelFile('append','_DSCVMethod_',FILENAME_TIME, image_name, modelsToUse,...
                        tout1 , tout2 , tout3 , tout4 , tout5 , 0.0 , 0.0 , 0.0 , 0.0 , ...
                        tout20, 0.0,individualResults.DSCV.totalTime);
                elseif (writeTimeExcel==1) % we create a new file
                    writeExcelFile('create','_DSCVMethod_',FILENAME_TIME, image_name, modelsToUse,...
                        tout1 , tout2 , tout3 , tout4 , tout5 , 0.0 , 0.0 , 0.0 , 0.0 , ...
                        tout20, 0.0,individualResults.DSCV.totalTime);
                    writeTimeExcel=2;
                elseif (writeTimeExcel==0)
                    %nothing
                end;
            end;
            
        end;
        if (sum(strcmp(segmentationMethod,'CV')) > 0)%% NOT SEEDED CV
            if strcmp(ImageModality,'ISH')
                weightLengthTerm = 0.01;
                sizeMedFilt = 5;
                lambda1 = 1;
                lambda2 = 1;
                %             minimumSize = 5000; % if 1000x1000 -> 25000
            elseif strcmp(ImageModality,'Lungs')
                %                 if image_name(1) == 'l' %Lungs
                weightLengthTerm = 0.01;
                sizeMedFilt = 3;
                lambda1 = 5;
                lambda2 = 2;
            elseif strcmp(ImageModality,'Knee')
                %                 elseif image_name(1) == 'r' %Knee
                weightLengthTerm = 0.01;
                sizeMedFilt = 3;
                lambda1 = 5;
                lambda2 = 2;
                %     end;
                %             minimumSize = 200;
            elseif strcmp(ImageModality,'MRI')
                weightLengthTerm = 0.01;
                sizeMedFilt = 1;
                lambda1 = 1;
                lambda2 = 1;
                %             minimumSize = 100; % WE DON'T REMOVE ANYTHING
            end;
            [tout18, newmaskCV] = ...
                segmentationCV(orig_im, out_dir_name,image_name, SOURCE_FOLDER,...
                REFERENCES,best_ref,timestamp,0, weightLengthTerm, sizeMedFilt,lambda1,lambda2,minimumSize,'CV');
            
            % time processing
            individualResults.CV.segmentationTime = tout18;
            individualResults.CV.totalTime = tout18 + tout1 + tout2 + tout3 + tout4 + tout5;
            
            if ValidateSegmentationResults==1
                if (referenceToUse > 0  | referenceToUse == -1)
                    useRegistrationToValidateResults = 0;
                else
                    useRegistrationToValidateResults = 1;
                end;
                [individualResults.CV.TP individualResults.CV.FP individualResults.CV.FN ...
                    individualResults.CV.DSC individualResults.CV.HD individualResults.CV.J] = ...
                    validateResults(newmaskCV,image_name,SOURCE_FOLDER,out_dir_name, show_output,'CV',...
                    useRegistrationToValidateResults,MODEL_TO_COMPARE,1);
            end;
            
            if (isunix==0)
                if (writeTimeExcel==2) %append
                    writeExcelFile('append','_CVMethod_',FILENAME_TIME, image_name, modelsToUse,...
                        tout1 , tout2 , tout3 , tout4 , tout5 , 0.0 , 0.0 , 0.0 , 0.0 , ...
                        tout18, 0.0,individualResults.CV.totalTime);
                elseif (writeTimeExcel==1) % we create a new file
                    writeExcelFile('create','_CVMethod_',FILENAME_TIME, image_name, modelsToUse,...
                        tout1 , tout2 , tout3 , tout4 , tout5 , 0.0 , 0.0 , 0.0 , 0.0 , ...
                        tout18, 0.0,individualResults.CV.totalTime);
                    writeTimeExcel=2;
                elseif (writeTimeExcel==0)
                    %nothing
                end;
            end;
        end;
    end;
    
    %% Segmentation of hippocampus - using Deformable Registration (B-splines)
    %% and Level Set Method (in this case, a new approach that takes into account edges (GAC), intensities (local Chan-Vese), and
    %% distance from the original registration result)
    if (sum(strcmp(segmentationMethod,'newHybrid')) > 0)
        orig_im = hip_img;
        %         load newmaskDS;
        if strcmp(ImageModality, 'ISH')
            gammaVFC = 2;
            lambda_in = 1;
            lambda_out = 1;
            weightCV = 2;
            weightPrior = 2;
            weightVFC = 1;
        elseif strcmp(ImageModality, 'MRI')
            gammaVFC = 1.5;
            lambda_in = 5;
            lambda_out = 1;
            weightCV = 5;
            weightPrior = 1;
            weightVFC = 0;
        elseif strcmp(ImageModality, 'Lungs')
            gammaVFC = 1.5;
            lambda_in = 5;
            lambda_out = 2;
            weightCV = 1;
            weightPrior = 0;
            weightVFC = 0;
        elseif strcmp(ImageModality, 'Knee')
            gammaVFC = 1.5;
            lambda_in = 2;
            lambda_out = 5;
            weightCV = 5;
            weightPrior = 1;
            weightVFC = 2;
        end;
        
        [tout23, newmaskNewHybrid] = ...
            segmentationNewHybrid(orig_im, out_dir_name,image_name, REFERENCES,best_ref,timestamp, ...
            gammaVFC,lambda_in,lambda_out,weightCV,weightPrior,weightVFC,ImageModality);
        
        % time processing
        individualResults.newHybrid.segmentationTime = tout23 + tout20;
        individualResults.newHybrid.totalTime = tout23 + tout20 + tout1 + tout2 + tout3 + tout4 + tout5;
        
        if ValidateSegmentationResults==1
            % VALIDATE RESULTS
            % If there's not a ground truth it will return 0.0 in all
            % parameters
            if (referenceToUse > 0 | referenceToUse == -1)
                useRegistrationToValidateResults = 0;
            else
                useRegistrationToValidateResults = 1;
            end;
            [individualResults.newHybrid.TP individualResults.newHybrid.FP individualResults.newHybrid.FN ...
                individualResults.newHybrid.DSC individualResults.newHybrid.HD individualResults.newHybrid.J] = ...
                validateResults(newmaskNewHybrid,image_name,SOURCE_FOLDER,out_dir_name, show_output,'newHybrid',...
                useRegistrationToValidateResults,MODEL_TO_COMPARE,1);
        end;
        
        if (isunix==0)
            if (writeTimeExcel==2) %append
                writeExcelFile('append','_newHybridMethod_',FILENAME_TIME, image_name, modelsToUse,...
                    tout1 , tout2 , tout3 , tout4 , tout5 , 0.0 , 0.0 , 0.0 , 0.0 , ...
                    tout20, 0.0,individualResults.newHybrid.totalTime);
            elseif (writeTimeExcel==1) % we create a new file
                writeExcelFile('create','_newHybridMethod_',FILENAME_TIME, image_name, modelsToUse,...
                    tout1 , tout2 , tout3 , tout4 , tout5 , 0.0 , 0.0 , 0.0 , 0.0 , ...
                    tout20, 0.0,individualResults.newHybrid.totalTime);
                writeTimeExcel=2;
            elseif (writeTimeExcel==0)
                %nothing
            end;
        end;
    end;
    
end;

fclose('all');

end



