%==========================================================================
%
%   Hybrid image segementation method
%
%--------------------------------------------------------------------------
%   Usage of varibles:
%   input: 
%       Img         = any gray/double/RGB input image
%       prior       = segmentation prior
%       num_iter    = total number of iterations
%       wCV         = weight of the C&V term
%       wPrior      = weight of the prior term
%       wEdge       = weight of the edge term
%       wLength          = weight of length term
%
%   output: 
%       phi        = updated level set function 
%
%--------------------------------------------------------------------------
%
% Description: 
%
%==========================================================================

function seg = hybrid_method(Img, prior, num_iter, wCV, wPrior, wEdge, wLength)
%%
%-- Default settings
%   length term wLength = 0.2
    if(~exist('wLength','var'))
        wLength=0.2;
    end

    %-- End default settings

    mask = maskcircle2(Img,'large');

    if size(Img,3)== 3
        P = rgb2gray(Img);
        P = double(P);
    else
        P = double(Img);
    end

    %-- End Initializations on input image Img and mask

    %%
    %--   Core function

    %Edge term init
    sigma=1.5;     % scale parameter in Gaussian kernel
    G = fspecial('gaussian',15,sigma);
    Img_smooth = conv2(P,G,'same');  % smooth image by Gaussiin convolution
    [Ix,Iy] = gradient(Img_smooth);
    f = Ix.^2+Iy.^2;
    g = 1./(1+f);  % edge indicator function.
    epsilon = 1.5;

%Debug
%     figure(); imshow(Img_smooth);
%     figure(); imshow(f);
%     figure(); imshow(g);

    %-- SDF
    %   Get the distance map of the initial mask

    mask = mask(:,:,1);
    mask(mask>=0.5) = 1;
    mask(mask<=0.5) = 0;

    phi0 = double_bwdist(mask) -double_bwdist(1-mask) + im2double(mask) -.5;
    %   initial force, set to eps to avoid division by zeros
    force = eps;

    %-- Display settings
    figure();
    subplot(3,3,1); imshow(Img); title('Input Image');
    subplot(3,3,2); contour(flipud(phi0), [0 0], 'r','LineWidth',1); title('initial contour');
    subplot(3,3,3); title('Segmentation');
    %-- End Display original image and mask

    %TO BE REMOVED
    L = P;
    PX = im2double(prior);

    [vx, vy] = gradient(g);
    figure(); show_force(vx.^2 + vy.^2);
    %-- Main loop
    for n=1:num_iter

        % Plain Chan & Vese
        force_image = 0; %
        inidx = find(phi0>=0);
        outidx = find(phi0<0);
        c1 = sum(sum(L.*Heaviside(phi0)))/(length(inidx)+eps); % average inside of Phi0
        c2 = sum(sum(L.*(1-Heaviside(phi0))))/(length(outidx)+eps); % verage outside of Phi0
        force_image = -(L-c1).^2+(L-c2).^2 + force_image;

        % Prior force
        % Computed as C&V on the prior image; currently without any
        % optimizations
        force_prior = 0;
        c1 = sum(sum(PX.*Heaviside(phi0)))/(length(inidx)+eps); % average inside of Phi0
        c2 = sum(sum(PX.*(1-Heaviside(phi0))))/(length(outidx)+eps); % verage outside of Phi0
        force_prior = -(PX-c1).^2+(PX-c2).^2 + force_prior;

        % Edges force
        % This simulates the two loops of the DRLSE implementation
        if(mod(n,10) == 0)
            [vx, vy] = gradient(g);
        end

        %phi0=NeumannBoundCond(phi0);
        [phi_x,phi_y]=gradient(phi0);
        figure(); show_force(phi_x.^2 + phi_y.^2); 

        s=sqrt(phi_x.^2 + phi_y.^2);
        smallNumber=1e-20;
        Nx=phi_x./(s+smallNumber); % add a small positive number to avoid division by zero
        Ny=phi_y./(s+smallNumber);
        figure(); show_force(Nx.^2 + Ny.^2)
        curvature=div(Nx,Ny);
        distRegTerm=distReg_p2(phi0);  % compute the distance regularization term in eqaution (13) with the double-well potential p2.
        diracPhi=Dirac(phi0,epsilon);
        %areaTerm=diracPhi.*g; % balloon/pressure force
        force_edge = diracPhi.*(vx.*Nx+vy.*Ny) + diracPhi.*g.*curvature;
        force_edge = distRegTerm
        
        % calculate the total force of the image
        force = wLength*kappa(phi0)./max(max(abs(kappa(phi0)))) + wCV*force_image + wPrior*force_prior + wEdge*force_edge;
        
        % normalized the force
        force = force./(max(max(abs(force))));
        
        subplot(3,3,4); show_force(force_image); title(strcat('CV term ', num2str(wCV)));
        subplot(3,3,5); show_force(force_prior); title(strcat('Prior term ', num2str(wPrior)));
        subplot(3,3,6); show_force(force_edge); title(strcat('Edge term ', num2str(wEdge)));
        subplot(3,3,7); show_force(force); title('Total force');

        % get stepsize dt
        dt = 5;

        % get parameters for checking whether to stop
        old = phi0;
        phi0 = phi0 + dt.*force;
        new = phi0;
        indicator = checkstop(old, new, dt);

        % intermediate output
        if(mod(n,3) == 0)
            showphi(Img, phi0, n);
        end;
%         if indicator % decide to stop or continue
%             showphi(Img, phi0, n);
% 
%             %make mask from SDF
%             seg = phi0<=0; %-- Get mask from levelset
% 
%             subplot(2,2,4); imshow(seg); title('Global Region-Based Segmentation');
% 
%             return;
%         end
    end;
    showphi(Img,phi0,n);

    seg = phi0 <= 0;

    subplot(3,3,9); imshow(seg); title('Global Region-Based Segmentation');
end

function f = distReg_p2(phi)
    % compute the distance regularization term with the double-well potential p2 in eqaution (16)
    [phi_x,phi_y] = gradient(phi);
    s = sqrt(phi_x.^2 + phi_y.^2);
    a = (s>=0) & (s<=1);
    b = (s>1);
    ps = a.*sin(2*pi*s)/(2*pi)+b.*(s-1);  % compute first order derivative of the double-well potential p2 in eqaution (16)
    dps = ((ps~=0).*ps+(ps==0))./((s~=0).*s+(s==0));  % compute d_p(s)=p'(s)/s in equation (10). As s-->0, wEdge have d_p(s)-->1 according to equation (18)
    f = div(dps.*phi_x - phi_x, dps.*phi_y - phi_y) + 4*del2(phi);  
end

function f = div(nx,ny)
    [nxx, junk] = gradient(nx);  
    [junk, nyy] = gradient(ny);
    f = nxx + nyy;
end

function f = Dirac(x, sigma)
    f = (1/2/sigma)*(1+cos(pi*x/sigma));
    b = (x<=sigma) & (x>=-sigma);
    f = f.*b;
end

function g = NeumannBoundCond(f)
    % Make a function satisfy Neumann boundary condition
    [nrow,ncol] = size(f);
    g = f;
    g([1 nrow],[1 ncol]) = g([3 nrow-2],[3 ncol-2]);  
    g([1 nrow],2:end-1) = g([3 nrow-2],2:end-1);          
    g(2:end-1,[1 ncol]) = g(2:end-1,[3 ncol-2]);  
end

function show_force(force)
        range = quantile(abs(force(:)), 0.99);
        imshow(force, [-range, range]);
        colormap(redgreencmap);
end