function [Friedman_pValue,c] ...
    = nonParametricMoreThanTwoMethods(M,significanceLevel,problemType,numberOfFunctions, ...
    methodUsed,outputPath,inputFile,numberOfMethodsToCompare, saveFigures, fileID, stringValues,GranadaCodesON)
%% MORE THAN TWO METHODS TO COMPARE
% When there are more than two groups to be compared, one must turn to other
% methods. If you cant meet the assumptions of ANOVA designs, you can use
% nonparametric methods to evaluate group differences. If the groups are
% independent, a KruskalWallis test will provide you with a useful approach.
% If the groups are dependent (for example, repeated measures or randomized
% block design), the Friedman test is more appropriate.

%% FRIEDMAN TEST
% friedman(X,reps) performs the nonparametric Friedman's test to
% compare the means of the columns of X. Friedman's test is similar
% to classical two-way ANOVA, but it tests only for column effects
% after adjusting for possible row effects.
% I think this test is more suitable when you have a matrix where every
% column is a method and every row a problem/dataset. In this case, we
% consider all rows as only one big function
[Friedman_pValue,t,st] = friedman(M,numberOfFunctions);
doPairwise = 0;
if Friedman_pValue < significanceLevel
    label = sprintf('Friedman test: There are significant differences between the medians of the methods');
    disp(label);
    doPairwise = 1;
else
    label = sprintf('Friedman test: There are NOT significant differences between the medians of the methods');
    disp(label);
end;
close all;
fprintf(fileID,'p-value: %d\n',Friedman_pValue);
fprintf(fileID,strcat(label,'\n'));
fprintf(fileID,'\n');

%% POST-HOC ANALYSIS: TUCKEY-KRAMER CORRECTION FOR Friedman test (NXN, no control method)
% multcompare(stats) performs a multiple comparison test using the
% information in the stats structure, and returns a matrix c of
% pairwise comparison results.
correction = 'tukey-kramer'; 
% This procedure is based on the Studentized range distribution. 
% It is optimal for balanced one-way ANOVA and similar procedures with 
% equal sample sizes. It has been proven to be conservative for one-way 
% ANOVA with different sample sizes. According to the unproven 
% Tukey-Kramer conjecture, it is also accurate for problems where the 
% quantities being compared are correlated, as in analysis of 
% covariance with unbalanced covariate values..
st.gnames = methodUsed';
if saveFigures
    [c,m,h,nms] = multcompare(st,'display','on','alpha',significanceLevel,'ctype',correction,'estimate','friedman');
else
    [c,m,h,nms] = multcompare(st,'display','off','alpha',significanceLevel,'ctype',correction,'estimate','friedman');
end;
if doPairwise
    % How to analyze results?
    % For example, if one row in c contains the following entries.
    % 2.0000  5.0000  1.9442  8.2206  14.4971
    % These numbers indicate that the mean of group 2 minus the mean
    % of group 5 is estimated to be 8.2206, and a 95% confidence
    % interval for the true difference of the means is [1.9442,
    % 14.4971]. In this example the confidence interval does not
    % contain 0.0, so the difference is significant at the 0.05 level.
    % If the confidence interval did contain 0.0, the difference would
    % not be significant at the 0.05 level.    
    differencesArray = ~(0.0 > c(1:size(c,1),3) & 0.0 < c(1:size(c,1),5));
    pairWise_StatisticalDifferences = methodUsed(c(differencesArray,1:2));
    label = sprintf('Pairwise Statistical Differences using Friedman test with the %s correction: ', correction);
    disp(label);
    fprintf(fileID,strcat(label,'\n'));
    temporal = c(differencesArray,1:2);
    if ~isempty(temporal)
        for i=1:size(pairWise_StatisticalDifferences,1)
            if problemType == 1
                [valuePairWise, indexPairWise]=sort([mean(M(:,temporal(i,1))),mean(M(:,temporal(i,2)))],'descend');
            else
                [valuePairWise, indexPairWise]=sort([mean(M(:,temporal(i,1))),mean(M(:,temporal(i,2)))],'ascend');
            end;
            label = sprintf('%s | %s -> %s',char(pairWise_StatisticalDifferences(i,1)),char(pairWise_StatisticalDifferences(i,2)),char(pairWise_StatisticalDifferences(i,indexPairWise(1))));
            disp(label)
            fprintf(fileID,strcat(label,'\n'));
        end;
    else
        label = sprintf('There are NOT pairwise differences between methods');
        disp(label);
        fprintf(fileID,strcat(label,'\n'));
    end;
    if saveFigures
        title(sprintf('Comparison Intervals Plot of the results obtained by each method using the %s correction',correction))
        saveas(gcf,strcat(outputPath,'ComparisonIntervalsPlot_',inputFile,'_Friedman.jpg'))
    end;
    close all;
    fprintf(fileID,'\n');
end;
fprintf(fileID,'\n');

%% KRUSKAL-WALLIS TEST
% KRUSKALWALLIS(X,GROUP,DISPLAYOPT) performs a non-parametric one-way
%     ANOVA to test the null hypothesis that independent samples from two or
%     more groups come from distributions with equal medians, and returns the
%     p-value for that test.  The data are assumed to come from continuous
%     distributions that are identical except possibly for location shifts
%     due to group effects, but are otherwise arbitrary.
[KruskalWallis_pValue,t,st] = kruskalwallis(M,methodUsed,'off');
doPairwise = 0;
if KruskalWallis_pValue < significanceLevel
    label = sprintf('Kruskal-Wallis test: There are significant differences between the medians of the methods');
    disp(label);
    doPairwise = 1;
else
    label = sprintf('Kruskal-Wallis test: There are NOT significant differences between the medians of the methods');
    disp(label);
end;
close all;
fprintf(fileID,'p-value: %d\n',KruskalWallis_pValue);
fprintf(fileID,strcat(label,'\n'));
fprintf(fileID,'\n');


%% POST-HOC ANALYSIS: DUNN-SIDAK CORRECTION FOR KRUSKAL-WALLIS (NXN, no control method)
% multcompare(stats) performs a multiple comparison test using the
% information in the stats structure, and returns a matrix c of
% pairwise comparison results.
correction = 'dunn-sidak'; %This procedure is similar to, but less conservative than, the Bonferroni procedure.
st.gnames = methodUsed';
if saveFigures
    [c,m,h,nms] = multcompare(st,'display','on','alpha',significanceLevel,'ctype',correction,'estimate','kruskalwallis');
else
    [c,m,h,nms] = multcompare(st,'display','off','alpha',significanceLevel,'ctype',correction,'estimate','kruskalwallis');
end;
if doPairwise
    % How to analyze results?
    % For example, if one row in c contains the following entries.
    % 2.0000  5.0000  1.9442  8.2206  14.4971
    % These numbers indicate that the mean of group 2 minus the mean
    % of group 5 is estimated to be 8.2206, and a 95% confidence
    % interval for the true difference of the means is [1.9442,
    % 14.4971]. In this example the confidence interval does not
    % contain 0.0, so the difference is significant at the 0.05 level.
    % If the confidence interval did contain 0.0, the difference would
    % not be significant at the 0.05 level.    
    differencesArray = ~(0.0 > c(1:size(c,1),3) & 0.0 < c(1:size(c,1),5));
    pairWise_StatisticalDifferences = methodUsed(c(differencesArray,1:2));
    label = sprintf('Pairwise Statistical Differences using Kruskal-Wallis test with the %s correction: ', correction);
    disp(label);
    fprintf(fileID,strcat(label,'\n'));
    temporal = c(differencesArray,1:2);
    if ~isempty(temporal)
        for i=1:size(pairWise_StatisticalDifferences,1)
            if problemType == 1
                [valuePairWise, indexPairWise]=sort([mean(M(:,temporal(i,1))),mean(M(:,temporal(i,2)))],'descend');
            else
                [valuePairWise, indexPairWise]=sort([mean(M(:,temporal(i,1))),mean(M(:,temporal(i,2)))],'ascend');
            end;
            label = sprintf('%s | %s -> %s',char(pairWise_StatisticalDifferences(i,1)),char(pairWise_StatisticalDifferences(i,2)),char(pairWise_StatisticalDifferences(i,indexPairWise(1))));
            disp(label)
            fprintf(fileID,strcat(label,'\n'));
        end;
    else
        label = sprintf('There are NOT pairwise differences between methods');
        disp(label);
        fprintf(fileID,strcat(label,'\n'));
    end;
    if saveFigures
        title(sprintf('Comparison Intervals Plot of the results obtained by each method using the %s correction',correction))
        saveas(gcf,strcat(outputPath,'ComparisonIntervalsPlot_',inputFile,'_kruskalwallis.jpg'))
    end;
    close all;
    fprintf(fileID,'\n');
    
    
%     %% FRIEDMAN TEST USING NEMENYI POST-HOC PROCEDURE
%     disp(' ')
%     disp('ALTERNATIVE CHECK 1: ')
%     labelMethods = char(methodUsed{1});
%     for j=2:size(methodUsed,2)
%         labelMethods = strcat(labelMethods,',',char(methodUsed{j}));
%     end;
%     [SIG FVAL CRITVAL RANKS CRITDIF] = testfriedman(M,'rows',significanceLevel,labelMethods);
%     if saveFigures
%         saveas(gcf,strcat(outputPath,'Friedman_NiemenyiPost-Hoc_',inputFile,'.jpg'))
%     end;
%     close all;
%     if SIG == 1
%         label = sprintf('Friedman test using Nemenyi Post-hoc: There are significant differences');
%         disp(label);
%     else
%         label = sprintf('Friedman test using Nemenyi Post-hoc: There are NOT significant differences');
%         disp(label);
%     end;
%     fprintf(fileID,strcat(label,'\n'));
%     label = sprintf(strcat('RANKS: ',labelMethods));
%     disp(label);
%     fprintf(fileID,strcat(label,'\n'));
%     label = sprintf(num2str(RANKS));
%     disp(label);
%     fprintf(fileID,strcat(label,'\n'));
%     fprintf(fileID,'\n');
    
    
    %% BONFERRONI-HOLM PROCEDURE FOR MULTIPLE WILCOXON SIGNED RANK TESTS
    disp(' ')
    disp('ALTERNATIVE CHECK 1: ')
    label = sprintf('Bonferroni-Holm procedure for multiple Wilcoxon signed rank tests');
    % Holms procedure can always be considered better than BonferroniDunns procedure, because it appropriately controls the
    % FWER (Family Wise Error Rate) and it is more powerful than BonferroniDunns procedure.
    disp(label);
    pvalues = [];
    temp = []; for meth = 1:numberOfMethodsToCompare temp=[temp meth]; end;
    allCombinations = nchoosek([temp],2);
    for index = 1:size(allCombinations,1)
        pvalues = [pvalues, signrank(M(:,allCombinations(index,1)),M(:,allCombinations(index,2)),'alpha',significanceLevel)];
    end;
    [corrected_p, h]=bonf_holm(pvalues,significanceLevel);
    disp('Test     Adjusted p-value   Comment')
    for index = 1:size(allCombinations,1)
        if corrected_p(index) < significanceLevel
            fprintf('%s-%s %11.4f             Reject Ho\n',int2str(allCombinations(index,1)),int2str(allCombinations(index,2)),corrected_p(index))
        else
            fprintf('%s-%s %11.4f             Fail to reject Ho\n',int2str(allCombinations(index,1)),int2str(allCombinations(index,2)),corrected_p(index))
        end
    end;
end;
fprintf(fileID,'\n');

if GranadaCodesON
    %% GRANADA CODES FOR MULTIPLE TESTS
    % Codes extracted from http://sci2s.ugr.es/sicidm/#nine
    % Reference papers:
    %  J. Derrac, S. Garca, D. Molina, F. Herrera, "A Practical Tutorial on the
    % Use of Nonparametric Statistical Tests as a Methodology for Comparing
    % Evolutionary and Swarm Intelligence Algorithms". Swarm and Evolutionary Computation 1:1 (2011) 3-18
    
    % This codes are NxN (no control method), and are used with files like
    % this (every column a method, every row a dataset):
    %     Dataset,C4.5,1NN,NaiveBayes,Kernel,CN2
    %     Cleveland,0.544,0.531,0.557,0.438,0.541
    %     Glass,0.674,0.736,0.720,0.355,0.704
    %     Abalone,0.218,0.201,0.249,0.164,0.260
    
    %     In 1N comparisons, a control
    % method is highlighted (the best performing algorithm) through
    % the application of the test. Then, all hypotheses of equality between
    % the control method and the rest can be tested by the application of
    % a set of post-hoc procedures. N  N comparisons, considering the
    % hypotheses of equality between all existing pairs of algorithms, are
    % also possible, with the inclusion of specific post-hoc procedures for
    % this task.
    
    disp(' ')
    disp('ALTERNATIVE CHECK 2: NxN (MULTIPLE TEST)')
    disp('Running statistical codes of the University of Granada')
    outputTEX = strcat('output_multipleTest',stringValues,'.tex');
    if strcmp(stringValues, 'All')
        csvFile = strcat(outputPath, inputFile, '.csv');
    else
        csvFile = strcat(outputPath,inputFile, '_',stringValues,'.csv');
    end;
    s = regexprep(mfilename('fullpath'), strcat('(\w*)',mfilename), '');
    destination = strcat(s,'UniGranada\multipleTest\');
    copyfile(csvFile,destination);
    originalFolder = pwd;
    try
        cd(destination)
        fileToRun = dir('*.csv'); % it has to be the only one CSV file in the statisticalTests folder
        command = ['java Friedman ',char(fileToRun.name),' > ',outputTEX];
        system(command);
        delete(fileToRun.name)
        command = ['pdfLatex ',outputTEX];
        system(command);
        fileToRun = dir('*.pdf');
        % IMPORTANT: when using only the statistical framework is important
        % to change these paths: 
        % strcat('../../../',outputPath) -> strcat('../../',outputPath)
        movefile(strcat('./',fileToRun.name),strcat('../../../',outputPath))
        fileToRun = dir('*.tex');
        movefile(strcat('./',fileToRun.name),strcat('../../../',outputPath))
        delete(strrep(outputTEX,'.tex','.aux'))
        delete(strrep(outputTEX,'.tex','.log'))
        cd(originalFolder)
        disp(' ')
        disp('The results (output.pdf) can be found in the globalResults folder')
        disp(' ')
    catch
        cd(originalFolder)
        disp(' ')
        disp('There was some error when running the Granada codes: it could have')
        disp('been during creating the tex file, during the conversion to pdf, ')
        disp('or when removing the aux and log files')
        disp(' ')
    end;
    
    disp(' ')
    disp('ALTERNATIVE CHECK 3: 1xN (CONTROL TEST)')
    disp('Running statistical codes of the University of Granada')
    outputTEX = strcat('output_controlTest',stringValues,'.tex');
    if strcmp(stringValues, 'All')
        csvFile = strcat(outputPath, inputFile, '.csv');
    else
        csvFile = strcat(outputPath,inputFile, '_',stringValues,'.csv');
    end;
    s = regexprep(mfilename('fullpath'), strcat('(\w*)',mfilename), '');
    destination = strcat(s,'UniGranada\controlTest\');
    copyfile(csvFile,destination);
    originalFolder = pwd;
    try
        cd(destination)
        fileToRun = dir('*.csv'); % it has to be the only one CSV file in the statisticalTests folder
        command = ['java Friedman ',char(fileToRun.name),' > ',outputTEX];
        system(command);
        delete(fileToRun.name)
        command = ['pdfLatex ',outputTEX];
        system(command);
        fileToRun = dir('*.pdf');
        movefile(strcat('./',fileToRun.name),strcat('../../../',outputPath))
        fileToRun = dir('*.tex');
        movefile(strcat('./',fileToRun.name),strcat('../../../',outputPath))
        delete(strrep(outputTEX,'.tex','.aux'))
        delete(strrep(outputTEX,'.tex','.log'))
        cd(originalFolder)
        disp(' ')
        disp('The results (output.pdf) can be found in the globalResults folder')
        disp(' ')
    catch
        cd(originalFolder)
        disp(' ')
        disp('There was some error when running the Granada codes: it could have')
        disp('been during creating the tex file, during the conversion to pdf, ')
        disp('or when removing the aux and log files')
        disp(' ')
    end;
end;


