
%% THIS FUNCTION RETURNS THE TP, FP AND FN RATES DOING THE VERIFICATION OF
%% SEGMENTATION RESULTS
function [TruePositivesPercentage FalsePositivesPercentage FalseNegativesPercentage DiceCoefficient Hausdorff JaccardIndex] = ...
    validateResults(newmask,image_name,sourceFolder,outFolder, show_output,label,useRegistration, MODEL_TO_COMPARE,saveImages)

% MODEL_TO_COMPARE = 'consensus'; % 'intersection','union','consensus','STAPLE' %
% STAPLE is an expectation-maximization algorithm for Simultaneous Truth And
% Performance Level Estimation. The algorithm considers a collection of segmentations and
% computes a probabilistic estimate of the hidden, implicit, true segmentation and a measure of the
% performance level achieved by each segmentation.
% The probabilistic segmentation that is estimated is a type of consensus
% segmentation, which is often used as a reference standard segmentation for assessing
% performance of different algorithms and when an improved segmentation is needed.
% In practice, it is quite similar to the consensus image.

% try
% INTERSECTION GROUND TRUTH IMAGE (contains the minimum segmented area
% in all manual segmentations) but now we want to use the CONSENSUS IMAGE
% (if one pixel has been segmented in, at least, three manual segmentations
% we can include it in the consensus image)
if isempty(findstr(image_name,'synthetic')) %synthetic only has consensus
    if strcmp(MODEL_TO_COMPARE,'consensus')
        load(strcat(sourceFolder,'/',image_name,'.mat'))
        intersection_image = new_image;
    elseif strcmp(MODEL_TO_COMPARE,'intersection')
        load(strcat(sourceFolder,'/',image_name,'_intersection.mat'))
    elseif strcmp(MODEL_TO_COMPARE,'union')
        load(strcat(sourceFolder,'/',image_name,'_union.mat'))
        intersection_image = union_image;
    elseif strcmp(MODEL_TO_COMPARE,'STAPLE')
        load(strcat(sourceFolder,'/',image_name,'_STAPLE.mat'))
        intersection_image = new_image;
    end;
else
    load(strcat(sourceFolder,'/',image_name,'.mat'))
    intersection_image = new_image;
end;
referenceSegmentation = intersection_image;
% automatically segmented image
automaticSegmentation = newmask;

% Control in the 'strange' case that the images are not binary
referenceSegmentation(referenceSegmentation >= 1) = 1;
referenceSegmentation(referenceSegmentation <=0) = 0;
referenceSegmentation(referenceSegmentation >= 0.5) = 1;
referenceSegmentation(referenceSegmentation < 0.5) = 0;
automaticSegmentation(automaticSegmentation >= 1) = 1;
automaticSegmentation(automaticSegmentation <=0) = 0;
automaticSegmentation(automaticSegmentation >= 0.5) = 1;
automaticSegmentation(automaticSegmentation < 0.5) = 0;

% for doing the registration both images has to have the same
% dimensions: we adapt the size of the floating/automatic image to the
% size of the reference image
referenceSize = size(referenceSegmentation);
automaticSize = size(automaticSegmentation);
differenceInRows = (referenceSize(1)-automaticSize(1))/2;
differenceInCols = (referenceSize(2)-automaticSize(2))/2;
moduloDifferenceInRows = mod((referenceSize(1)-automaticSize(1)),2);
moduloDifferenceInCols = mod((referenceSize(2)-automaticSize(2)),2);
if referenceSize(1) > automaticSize(1)
    automaticSegmentation = [zeros(floor(abs(differenceInRows)),size(automaticSegmentation,2));automaticSegmentation; zeros(floor(abs(differenceInRows))+moduloDifferenceInRows,size(automaticSegmentation,2))];
elseif referenceSize(1) < automaticSize(1)
    automaticSegmentation(referenceSize(1)+1:referenceSize(1)+floor(abs(differenceInRows)),:) = [];
    automaticSegmentation(1:floor(abs(differenceInRows))+moduloDifferenceInRows,:) = [];
end;
if referenceSize(2) > automaticSize(2)
    automaticSegmentation = [zeros(referenceSize(1),floor(abs(differenceInCols))),automaticSegmentation, zeros(referenceSize(1),floor(abs(differenceInCols))+moduloDifferenceInCols)];
elseif referenceSize(2) < automaticSize(2)
    automaticSegmentation(:,referenceSize(2)+1:referenceSize(2)+floor(abs(differenceInCols))) = [];
    automaticSegmentation(:,1:floor(abs(differenceInCols))+moduloDifferenceInCols) = [];
end;

percentages = {};
referImages = {};
autoImages = {};
%% REGISTRATION TO VALIDATE RESULTS
% Sometimes the rigid registration is awful, even with very good
% segmentations. I think it's because it's no able to find the
% tranformation for aligning with the reference. So, we can have a good segmentation but a bad
% TP percentage due to this problem. For solving this, we try 4
% registrations (for avoiding problems with the initial seed value, and the
% lower and upper bounds).
if useRegistration == 1
    trials = 1;
    while trials < 4
        % we're going to try with different image size, so we need to store
        % the original size in order to come back to it
        oldSizeReferenceSegmentation = size(referenceSegmentation);
        oldSizeAutomaticSegmentation = size(automaticSegmentation);
        newReferenceSegmentation = imresize(referenceSegmentation, [round(size(referenceSegmentation,1)/(trials+1)) round(size(referenceSegmentation,2)/(trials+1))],'nearest');
        newAutomaticSegmentation = imresize(automaticSegmentation, [round(size(automaticSegmentation,1)/(trials+1)) round(size(automaticSegmentation,2)/(trials+1))],'nearest');
        automaticSegmentation = newAutomaticSegmentation;
        referenceSegmentation = newReferenceSegmentation;
        I2 = referenceSegmentation; % I2 will be the image used to calculate the registration error
        I1 = automaticSegmentation; % I1 will be the image which is rigidly transformed
        
        % REGISTRATION WITH LEVENBERG-MARQUARDT
        % Rigid vector of length 3 -> [translateX translateY rotate]
        lbg=[-250  -250  -0.314]; %lbg=[-250  -250  -0.25];
        ubg=[250  250  0.314]; % ubg=[250  250  0.25];
        type='sd';
        Xcurrent = [0.5 0.5 0.5];
        scale = [1 1 1];
        display_lm = 'off';
        itermax_lm = 2000;
        doAffine(0,I1, I2, lbg, ubg, type); %initialize by passing the two images, the boundaries and the distance method
        [xmin tfxmin]=lsqnonlin(@(x)affine_mask(x,scale,I1,I2,type,lbg,ubg),Xcurrent,[],[],optimset('Display',display_lm,'TolX',0,'MaxIter',30000,'TolFun',0,'MaxFunEvals', itermax_lm, 'Algorithm','levenberg-marquardt'));
        xmin = xmin.*(ubg-lbg) + lbg;
        M=make_transformation_matrix(xmin(1:2),xmin(3));
        Icor=affine_transform(I1,M,0);
        newIcor = Icor;
        % the followin two lines are needed because "affine_transform" does
        % not interpolate properly: even if the interpolation is the
        % nearest neighbour it will have intermediate levels of grey, and
        % we don't want that.
        newIcor(newIcor<0.2) = 0;
        newIcor(newIcor>=0.2) = 1;
        % now the transformed image substitutes the previous automatic
        % segmentation: since we're using only rigid transformations the
        % image is the same
        automaticSegmentation = newIcor;
        clear I1 I2
        % we recover the original size in order to calculate the TP rate
        newReferenceSegmentation = imresize(referenceSegmentation, [oldSizeReferenceSegmentation(1) oldSizeReferenceSegmentation(2)],'nearest');
        newAutomaticSegmentation = imresize(automaticSegmentation, [oldSizeAutomaticSegmentation(1) oldSizeAutomaticSegmentation(2)],'nearest');
        referenceSegmentation = newReferenceSegmentation;
        automaticSegmentation = newAutomaticSegmentation;
        % we save the referenceImages and the automaticImages used in every
        % iteration. We save the reference because maybe in the process of
        % reducing size and enlarging it changes a little bit, and I want
        % to have the exact image with what we achieve the TP percentage
        referImages{trials} = referenceSegmentation;
        autoImages{trials} = automaticSegmentation;
        % we calculate the percentage
        intersection = referenceSegmentation + automaticSegmentation;
        intersection(intersection<2)=0;intersection(intersection>1)=1;
        imageArea = size(intersection,1) * size(intersection,2);
        %         AreaSegmentedAutomatically = (sum(sum(automaticSegmentation))/imageArea)*100;
        %         AreaSegmentedManually = (sum(sum(referenceSegmentation))/imageArea)*100;
        %         TruePositivesPercentage = (sum(sum(intersection))/sum(sum(referenceSegmentation)))*100;
        
        union = referenceSegmentation + automaticSegmentation;
        union(union>0)=1; union(union <= 0)=0;
        
        DiceCoefficient = ((2*sum(sum(intersection)))/(sum(sum(intersection))+sum(sum(union))));
        
        percentages{trials} = DiceCoefficient %TruePositivesPercentage
        % if the percentage is bigger than the 80% there're a lot of
        % posibilites of having done a good registration
        if percentages{trials} > 0.75 %80
            break;
        end;
        trials = trials + 1;
    end; %endWhile
    
    
    %% WE CAN TRY WITH GA in the worst possible case: if there are no
    %% percentages higher than 55% there are a lot of possibilities of
    %% having done a bad registration
    if (sum([percentages{:}]>0.65) == 0) %55
        trials = 2;
        %while trials < 4
        referenceSegmentation = intersection_image;
        automaticSegmentation = newmask;
        
        referenceSize = size(referenceSegmentation);
        automaticSize = size(automaticSegmentation);
        differenceInRows = (referenceSize(1)-automaticSize(1))/2;
        differenceInCols = (referenceSize(2)-automaticSize(2))/2;
        moduloDifferenceInRows = mod((referenceSize(1)-automaticSize(1)),2);
        moduloDifferenceInCols = mod((referenceSize(2)-automaticSize(2)),2);
        if referenceSize(1) > automaticSize(1)
            automaticSegmentation = [zeros(floor(abs(differenceInRows)),size(automaticSegmentation,2));automaticSegmentation; zeros(floor(abs(differenceInRows))+moduloDifferenceInRows,size(automaticSegmentation,2))];
            
        elseif referenceSize(1) < automaticSize(1)
            
            automaticSegmentation(referenceSize(1)+1:referenceSize(1)+floor(abs(differenceInRows)),:) = [];
            automaticSegmentation(1:floor(abs(differenceInRows))+moduloDifferenceInRows,:) = [];
        end;
        if referenceSize(2) > automaticSize(2)
            automaticSegmentation = [zeros(referenceSize(1),floor(abs(differenceInCols))),automaticSegmentation, zeros(referenceSize(1),floor(abs(differenceInCols))+moduloDifferenceInCols)];
        elseif referenceSize(2) < automaticSize(2)
            
            automaticSegmentation(:,referenceSize(2)+1:referenceSize(2)+floor(abs(differenceInCols))) = [];
            automaticSegmentation(:,1:floor(abs(differenceInCols))+moduloDifferenceInCols) = [];
        end;
        oldSizeReferenceSegmentation = size(referenceSegmentation);
        oldSizeAutomaticSegmentation = size(automaticSegmentation);
        newReferenceSegmentation = imresize(referenceSegmentation, [round(size(referenceSegmentation,1)/trials) round(size(referenceSegmentation,2)/trials)],'nearest');
        newAutomaticSegmentation = imresize(automaticSegmentation, [round(size(automaticSegmentation,1)/trials) round(size(automaticSegmentation,2)/trials)],'nearest');
        
        automaticSegmentation = newAutomaticSegmentation;
        referenceSegmentation = newReferenceSegmentation;
        
        I2 = referenceSegmentation; % I2 will be the image used to calculate the registration error
        I1 = automaticSegmentation; % I1 will be the image which is rigidly transformed
        
        % Rigid vector of length 3 -> [translateX translateY rotate]
        lbg=[round(-700/trials)  round(-200/trials)  round(-0.50/trials)]; %lbg=[-250  -250  -0.25];
        ubg=[round(700/trials)  round(200/trials)  round(0.50/trials)]; % ubg=[250  250  0.25];
        type='sd';
        
        scale = [1 1 1];
        options = gaoptimset;
        %options.Display = 'iter';
        options.PopulationSize = 100;
        options.Generations = 20;
        options.TolFun = 0;
        options.CrossoverFraction = 0.8;
        options.CrossoverFunction = @crossovertwopoint;
        options.EliteCount = 3;
        options.MutationFcn = {@mutationuniform, 0.2};
        options.HybridFcn = @fminsearch;
        options.SelectionFcn = {@selectiontournament,2};
        options.CreationFcn = @gacreationlinearfeasible;
        problem.nvars = 3;
        problem.lb = [0 0 0];
        problem.ub = [1 1 1];
        problem.fitnessfcn = @(x)affine_mask(x,scale,I1,I2,type,lbg,ubg);
        problem.options = options;
        
        tfxmin = 800;
        for i=1:3
            [xmint tfxmint] = ga(problem);%try also with lbg, ubg
            if tfxmint < tfxmin
                xmin = xmint;
                tfxmin = tfxmint;
                
                xmin = xmin.*(ubg-lbg) + lbg;
                
                M=make_transformation_matrix(xmin(1:2),xmin(3));
                
                Icor2=affine_transform(I1,M,0); % 3 stands for cubic interpolation
                newIcor2 = Icor2;
                newIcor2(newIcor2<0.2) = 0;
                newIcor2(newIcor2>=0.2) = 1;
                automaticSegmentation2 = newIcor2;
                newReferenceSegmentation2 = imresize(referenceSegmentation, [oldSizeReferenceSegmentation(1) oldSizeReferenceSegmentation(2)],'nearest');
                newAutomaticSegmentation2 = imresize(automaticSegmentation2, [oldSizeAutomaticSegmentation(1) oldSizeAutomaticSegmentation(2)],'nearest');
                referenceSegmentation2 = newReferenceSegmentation2;
                automaticSegmentation2 = newAutomaticSegmentation2;
                referImages{size(referImages,2)+1} = referenceSegmentation2;
                autoImages{size(autoImages,2)+1} = automaticSegmentation2;
                intersection2 = referenceSegmentation2 + automaticSegmentation2;
                intersection2(intersection2<2)=0;intersection2(intersection2>1)=1;
                %                 TruePositivesPercentage2 = (sum(sum(intersection2))/sum(sum(referenceSegmentation2)))*100;
                union2 = referenceSegmentation2 + automaticSegmentation2;
                union2(union2>0)=1; union2(union2 <= 0)=0;
                DiceCoefficient2 = ((2*sum(sum(intersection2)))/(sum(sum(intersection2))+sum(sum(union2))));
                percentages{size(percentages,2)+1} = DiceCoefficient2   %TruePositivesPercentage2
                if percentages{size(percentages,2)} > 0.75 %80
                    break;
                end;
                
            end %endif
            %     disp(i)
        end %endfor
        %             if percentages{size(percentages,2)} > 80
        %                 break;
        %             end;
        %trials = trials +1;
        
        %end; %while
    end;
    
    clear new_image intersection_image union_image
    
    % Between all the values we have calculated we will keep not the
    % biggest, since the biggest is going to be obtained always with
    % smaller images sizes (deforming more the image, with a bigger
    % bias), but with the one with the bigger gap between itself and
    % the closer iteration.
    if size(percentages,2)~=1
        differencesSize = [];
        for indice=1:size(percentages,2)-1
            differencesSize = [differencesSize abs(percentages{indice}-percentages{indice+1})];
        end;
        [maxDifference bestTrial] = max(differencesSize);
        [value bestTrial2]= max([percentages{bestTrial},percentages{bestTrial+1}]);
        if bestTrial2==1
            referenceSegmentation = referImages{bestTrial};
            automaticSegmentation = autoImages{bestTrial};
        else
            referenceSegmentation = referImages{bestTrial+1};
            automaticSegmentation = autoImages{bestTrial+1};
        end;
    else
        referenceSegmentation = referImages{1};
        automaticSegmentation = autoImages{1};
    end;
    clear referImages autoImages
end;

intersection = referenceSegmentation + automaticSegmentation;
intersection(intersection<2)=0;intersection(intersection>1)=1;

union = referenceSegmentation + automaticSegmentation;
union(union>0)=1; union(union <= 0)=0;

imageArea = size(intersection,1) * size(intersection,2);
AreaSegmentedAutomatically = (sum(sum(automaticSegmentation))/imageArea)*100;
AreaSegmentedManually = (sum(sum(referenceSegmentation))/imageArea)*100;
TruePositivesPercentage = (sum(sum(intersection))/sum(sum(referenceSegmentation)))*100;
TP = (sum(sum(intersection))/imageArea)*100; % TP with respect to the whole image. It can be used to calculate Dice Coefficient
percentage = TruePositivesPercentage;

NotSegmentedParts = referenceSegmentation-automaticSegmentation;
NotSegmentedParts(NotSegmentedParts <= 0) = 0;
NotSegmentedParts(NotSegmentedParts > 0) = 1;

FalseNegativesPercentage =  (sum(sum(NotSegmentedParts))/sum(sum(referenceSegmentation)))*100; % It should be the same as: 100 - TruePositivesPercentage
FN = (sum(sum(NotSegmentedParts))/imageArea)*100; % FN with respect to the whole image. It can be used to calculate Dice Coefficient

RGBNotSegmentedParts = repmat(uint8(255.*NotSegmentedParts),[1 1 3]);
RGBNotSegmentedParts(:,:,2) = 0;
RGBNotSegmentedParts(:,:,3) = 0;

SegmentingErrors = referenceSegmentation-automaticSegmentation;
SegmentingErrors(SegmentingErrors > 0) = 0;
SegmentingErrors(SegmentingErrors < 0) = 1;

FalsePositivesPercentage = (sum(sum(SegmentingErrors))/sum(sum(automaticSegmentation)))*100;
FP = (sum(sum(SegmentingErrors))/imageArea)*100; % FP with respect to the whole image. It can be used to calculate Dice Coefficient

% Dice Similarity Coefficient can be calculated in some of the
% following manners:
% (2*intersection)/(intersection + union)
% (2*TP)/(FP+TP+TP+FN)
% (2*intersection)/(reference + automatic)
DiceCoefficient = ((2*sum(sum(intersection)))/(sum(sum(intersection))+sum(sum(union))));
% Jaccard Similarity Index
% Dice/(2-Dice)
% intersection/union
JaccardIndex = DiceCoefficient/(2-DiceCoefficient);

RGBSegmentingErrors = repmat(uint8(255.*SegmentingErrors),[1 1 3]);
RGBSegmentingErrors(:,:,1) = 0;
RGBSegmentingErrors(:,:,3) = 0;

RGBintersection = repmat(uint8(255.*intersection),[1 1 3]);
result = RGBintersection + RGBNotSegmentedParts + RGBSegmentingErrors;

result1 = edge(imresize(referenceSegmentation, [round(size(referenceSegmentation,1)/5) round(size(referenceSegmentation,2)/5)]), 'sobel');
result2 = edge(imresize(automaticSegmentation, [round(size(automaticSegmentation,1)/5) round(size(automaticSegmentation,2)/5)]), 'sobel');
[row1, col1, V1]=find(result1==1);
[row2, col2, V2]=find(result2==1);

% The  Hausdorff  distance represents a  measure of the
% spatial distance between two sets of points. It represents
% the farthest of the nearest point for each pixel in the
% boundary of the reference and the automatic segmentation.

if isempty(row2) % If nothing comes segmented --> maximum distance (diagonal of the image)
    Hausdorff = (sqrt((size(result1,1))^2 + (size(result1,2))^2))*5;
else
    minDists1 = zeros(1,size(row1,1));
    for ii=1:size(row1,1)
        dists = zeros(1,size(row2,1));
        for j=1:size(row2,1)
            dist = sqrt((row1(ii)-row2(j))^2 + (col1(ii)-col2(j))^2);
            dists(j) = dist;
        end;
        minDists1(ii) = min(dists);
    end;
    
    minDists2 = zeros(1,size(row2,1));
    for ii=1:size(row2,1)
        dists = zeros(1,size(row1,1));
        for j=1:size(row1,1)
            dist = sqrt((row2(ii)-row1(j))^2 + (col2(ii)-col1(j))^2);
            dists(j) = dist;
        end;
        minDists2(ii) = min(dists);
    end;
    % Here, we multiply the result per 5 since the images, to
    % calculate the boundaries, were resized with a factor of 5.
    Hausdorff = (max([minDists1,minDists2]))*5;
end;

if (show_output > 0)
    figure,colormap(hot),imshow(result),
    textTitle = sprintf('White: parts correctly segmented | Red: parts should have been segmented | Green: parts should have NOT been segmented. Dice: %s%%',num2str(DiceCoefficient));
    title(textTitle)
    saveas(gcf,strcat(outFolder,'/',image_name,'_',label,'_overlapped.fig'));
end;
if saveImages
    outImage = sprintf('%s/%s_%s_overlapped.png',outFolder,image_name,label);
    imwrite(result, outImage,'png');
    out_file_name = sprintf('%s/%s_%s_segmentationValues.txt',outFolder,image_name,label);
    f = fopen(out_file_name,'w');
    fprintf(f,'Hausdorff: %f\n\nDiceCoefficient: %f\n\nJaccardIndex: %f\n\nTruePositivesPercentage (AutomaticSegmentation and GroundTruth): %f\n\nFalsePositivesPercentage (percentage of the automatically segmented image that should not have been segmented): %f\n\nFalseNegativesPercentage (percentage of ground truth, that should have been segmented but it was not): %f\n', ...
        Hausdorff, DiceCoefficient, JaccardIndex, TruePositivesPercentage, FalsePositivesPercentage, FalseNegativesPercentage);
end;
% catch TruePositivesPercentage
%     TruePositivesPercentage = 0.0;
%     FalseNegativesPercentage = 0.0;
%     FalsePositivesPercentage = 0.0;
%     out_file_name = sprintf('%s/%s.txt',outFolder,image_name);
%     f = fopen(out_file_name,'w');
%     fprintf(f,'Some error has happened. Maybe there is not ground truth associated to this image.\nTruePositivesPercentage: %f\n',TruePositivesPercentage);
%
% end;

clear referenceSegmentation automaticSegmentation NotSegmentedParts RGBNotSegmentedParts SegmentingErrors RGBSegmentingErrors RGBintersection

end





