function [S, MG, Nmax]=seg_fuzzy(I,smooth,Nm)
%
% SEG_FUZZY  Fuzzy thresholding segmentation
%
% INPUT
%		I 	input image
%		smooth	(=1) gaussian smoothing of data
%		Nm:	normalization
%			=1 absolute max
%			=2 relative max                  
%
% OUTPUT
%         	S:      Output value
%		MG:	Local Agregation (based on median)
%		Nmax    Number of maxima in histogram
%                       (Number of fuzzy sets)
%
% Implementation done up to 5 different output sets. If histogram shows more
% than 5 maxima reduced to the top 5. In this case output Nmax has two values:
%
%  Nmax=[Real_Nmax, 5]
%
% Algorithm proposed in:
%
%       Santiago Aja-Fernández, Gonzalo Vegas-Sánchez-Ferrero, 
%       Miguel A. Martín Fernández, Soft thresholding for medical 
%       image segmentation, EMBC'2010, Buenos Aires, Sept. 2010.
%
% EXAMPLE
%
%     [S,MG,Nmax]=seg_fuzzy(I,1,2);
%
%
% Santiago Aja-Fernandez, LPI V2.0
% www.lpi.tel.uva.es/~santi
% Valladolid, 
% Original: 20/04/2010,

I=mean(double(I),3);
%
%Smooth original image for better thresholding
if smooth==1 
  I2=I;
  I2b=[I2(1,:); I2(1,:);I2(1,:);I2(1,:);I2(1,:);
       I2;
       I2(end,:); I2(end,:);I2(end,:);I2(end,:);I2(end,:)];
       I2b=[I2b(:,1) I2b(:,1) I2b(:,1) I2b(:,1) I2b(:,1) I2b I2b(:,end) I2b(:,end) I2b(:,end) I2b(:,end) I2b(:,end)];
   %I3=filter2(ones(5)/25,I2b);
  window = fspecial('gaussian', 11, 1.5);
  I3=filter2(window,I2b);
  I3=I3(6:end-5,6:end-5);
  I=I3;
end

%Normalization-------------
if Nm==1
   Mw=max(I(:));
else
  S1=std(I(:));              
  M1=mean(I(:));
  Mw=max(I((I>(M1-2.*S1))&(I<(M1+2.*S1))));
end
I=I./Mw;


%Histogram of the Image
[h1,x]=hist(I(:),100);
h2=conv(ones([7,1])/7,h1);
h=h2(4:(end-3));
h=(h./sum(h))./(x(2)-x(1));
Dr=diff(h);
M=[Dr(1:end-1).*Dr(2:end)];
Maxt=find(double(M<0).*(Dr(1:end-1)>0))+1;
Nmax=length(Maxt); %(Number of fuzzy sets)
%MAX search

if Nmax==3

  %Modeling the Histogram as a Gaussian mixture

  fu_x=@(y)sum((-h+y(1).*normpdf(x,y(3),y(4))+y(2).*normpdf(x,y(5),y(6))+y(7).*normpdf(x,y(8),y(9))).^2);
  X=fminsearch(fu_x,[0.3,0.3,x(Maxt(1)),0.1,x(Maxt(2)),0.1,0.3,x(Maxt(3)),0.1]);

  C1=normpdf(x,X(3),X(4));
  Mc1=max(C1);
  C1=C1./Mc1;
  C2=normpdf(x,X(5),X(6));
  Mc2=max(C2);
  C2=C2./Mc2;
  C3=normpdf(x,X(8),X(9));
  Mc3=max(C3);
  C3=C3./Mc3;
  %Membership of data (image) to each of the classes

  Ac1=Mc1.*normpdf(I,X(3),X(4));
  Ac2=Mc2.*normpdf(I,X(5),X(6));
  Ac3=Mc3.*normpdf(I,X(8),X(9));

  TM(:,:,1)=Ac1;
  TM(:,:,2)=Ac2;
  TM(:,:,3)=Ac3;
  [Vx Mx]=max(TM,[],3);%AGREGRATION by MAX


elseif Nmax==2
fu_x=@(y)sum((-h+y(1).*normpdf(x,y(3),y(4))+y(2).*normpdf(x,y(5),y(6))).^2);
X=fminsearch(fu_x,[0.3,0.3,x(Maxt(1)),0.1,x(Maxt(2)),0.1,0.3]);
C1=normpdf(x,X(3),X(4));
Mc1=max(C1);
C1=C1./Mc1;
C2=normpdf(x,X(5),X(6));
Mc2=max(C2);
C2=C2./Mc2;

  %Membership of data (image) to each of the classes
Ac1=Mc1.*normpdf(I,X(3),X(4));
Ac2=Mc2.*normpdf(I,X(5),X(6));
TM(:,:,1)=Ac1;
TM(:,:,2)=Ac2;
[Vx Mx]=max(TM,[],3);%AGREGRATION by MAX



elseif Nmax==4

 %Modeling the Histogram as a Gaussian mixture

  fu_x=@(y)sum((-h+y(1).*normpdf(x,y(3),y(4))+y(2).*normpdf(x,y(5),y(6))+y(7).*normpdf(x,y(8),y(9))+y(10).*normpdf(x,y(11),y(12))).^2);
  X=fminsearch(fu_x,[0.25,0.25,x(Maxt(1)),0.1,x(Maxt(2)),0.1,0.3,x(Maxt(3)),0.1,0.25,Maxt(4),0.1]);
  

  C1=normpdf(x,X(3),X(4));
  Mc1=max(C1);
  C1=C1./Mc1;
  C2=normpdf(x,X(5),X(6));
  Mc2=max(C2);
  C2=C2./Mc2;
  C3=normpdf(x,X(8),X(9));
  Mc3=max(C3);
  C3=C3./Mc3;
  C4=normpdf(x,X(11),X(12));
  Mc4=max(C4);
  C4=C4./Mc4;

  %Membership of data (image) to each of the classes

  Ac1=Mc1.*normpdf(I,X(3),X(4));
  Ac2=Mc2.*normpdf(I,X(5),X(6));
  Ac3=Mc3.*normpdf(I,X(8),X(9));
  Ac4=Mc4.*normpdf(I,X(11),X(12));

  TM(:,:,1)=Ac1;
  TM(:,:,2)=Ac2;
  TM(:,:,3)=Ac3;
  TM(:,:,4)=Ac4;

  [Vx Mx]=max(TM,[],3); %AGREGRATION by MAX




elseif Nmax==5

  fu_x=@(y)sum((-h+y(1).*normpdf(x,y(3),y(4))+y(2).*normpdf(x,y(5),y(6))+y(7).*normpdf(x,y(8),y(9))+y(10).*normpdf(x,y(11),y(12))+y(13).*normpdf(x,y(14),y(15))).^2);
  X=fminsearch(fu_x,[0.25,0.25,x(Maxt(1)),0.1,x(Maxt(2)),0.1,0.3,x(Maxt(3)),0.1,0.25,Maxt(4),0.1,0.25,Maxt(5),0.1]);
  
  C1=normpdf(x,X(3),X(4));
  Mc1=max(C1);
  C1=C1./Mc1;
  C2=normpdf(x,X(5),X(6));
  Mc2=max(C2);
  C2=C2./Mc2;
  C3=normpdf(x,X(8),X(9));
  Mc3=max(C3);
  C3=C3./Mc3;
  C4=normpdf(x,X(11),X(12));
  Mc4=max(C4);
  C4=C4./Mc4;
  C5=normpdf(x,X(14),X(15));
  Mc5=max(C5);
  C5=C5./Mc5;

  %Membership of data (image) to each of the classes
  Ac1=Mc1.*normpdf(I,X(3),X(4));
  Ac2=Mc2.*normpdf(I,X(5),X(6));
  Ac3=Mc3.*normpdf(I,X(8),X(9));
  Ac4=Mc4.*normpdf(I,X(11),X(12));
 Ac5=Mc5.*normpdf(I,X(14),X(15));

  TM(:,:,1)=Ac1;
  TM(:,:,2)=Ac2;
  TM(:,:,3)=Ac3;
  TM(:,:,4)=Ac4;
  TM(:,:,5)=Ac4;  
  [Vx Mx]=max(TM,[],3);%AGREGRATION by MAX
elseif Nmax==1
error('Only one maximum find. Segmentation is no possible');
elseif Nmax>5
Nmax=[8,5];
[A,B]=sort(Dr(Maxt));
Maxt=Maxt(B);
Maxt=Maxt(1:5);
  fu_x=@(y)sum((-h+y(1).*normpdf(x,y(3),y(4))+y(2).*normpdf(x,y(5),y(6))+y(7).*normpdf(x,y(8),y(9))+y(10).*normpdf(x,y(11),y(12))+y(13).*normpdf(x,y(14),y(15))).^2);
  X=fminsearch(fu_x,[0.25,0.25,x(Maxt(1)),0.1,x(Maxt(2)),0.1,0.3,x(Maxt(3)),0.1,0.25,Maxt(4),0.1,0.25,Maxt(5),0.1]);
  
  C1=normpdf(x,X(3),X(4));
  Mc1=max(C1);
  C1=C1./Mc1;
  C2=normpdf(x,X(5),X(6));
  Mc2=max(C2);
  C2=C2./Mc2;
  C3=normpdf(x,X(8),X(9));
  Mc3=max(C3);
  C3=C3./Mc3;
  C4=normpdf(x,X(11),X(12));
  Mc4=max(C4);
  C4=C4./Mc4;
  C5=normpdf(x,X(14),X(15));
  Mc5=max(C5);
  C5=C5./Mc5;

  %Membership of data (image) to each of the classes
  Ac1=Mc1.*normpdf(I,X(3),X(4));
  Ac2=Mc2.*normpdf(I,X(5),X(6));
  Ac3=Mc3.*normpdf(I,X(8),X(9));
  Ac4=Mc4.*normpdf(I,X(11),X(12));
 Ac5=Mc5.*normpdf(I,X(14),X(15));

  TM(:,:,1)=Ac1;
  TM(:,:,2)=Ac2;
  TM(:,:,3)=Ac3;
  TM(:,:,4)=Ac4;
  TM(:,:,5)=Ac4;  
  [Vx Mx]=max(TM,[],3);%AGREGRATION by MAX

end

%--------------------------------------


Mx2(:,:,1)=Mx;
Mx2(:,:,2)=shiftmat(Mx,[-1,-1]);
Mx2(:,:,3)=shiftmat(Mx,[-1,0]);
Mx2(:,:,4)=shiftmat(Mx,[-1,1]);
Mx2(:,:,5)=shiftmat(Mx,[1,-1]);
Mx2(:,:,6)=shiftmat(Mx,[1,0]);
Mx2(:,:,7)=shiftmat(Mx,[1,1]);
Mx2(:,:,8)=shiftmat(Mx,[0,-1]);
Mx2(:,:,9)=shiftmat(Mx,[0,1]);
MG=median(Mx2,3);
%Mh=sum((Mx2==repmat(MG,[1,1,9])),3);

% S=Mx;
S = MG;
